from mayan.apps.common.tests.base import GenericViewTestCase

from ..models import WorkflowState
from ..permissions import permission_workflow_edit, permission_workflow_view

from .literals import (
    TEST_WORKFLOW_STATE_LABEL, TEST_WORKFLOW_STATE_COMPLETION,
)
from .mixins import WorkflowStateViewTestMixin, WorkflowTestMixin


class WorkflowStateViewTestCase(
    WorkflowTestMixin, WorkflowStateViewTestMixin, GenericViewTestCase
):
    def test_create_workflow_state_no_access(self):
        self._create_test_workflow()

        response = self._request_test_workflow_state_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowState.objects.count(), 0)

    def test_create_workflow_state_with_access(self):
        self._create_test_workflow()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_state_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowState.objects.count(), 1)
        self.assertEqual(
            WorkflowState.objects.all()[0].label, TEST_WORKFLOW_STATE_LABEL
        )
        self.assertEqual(
            WorkflowState.objects.all()[0].completion,
            TEST_WORKFLOW_STATE_COMPLETION
        )

    def test_create_workflow_state_invalid_completion_with_access(self):
        self._create_test_workflow()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_state_create_view(
            extra_data={'completion': ''}
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowState.objects.count(), 1)
        self.assertEqual(
            WorkflowState.objects.all()[0].label, TEST_WORKFLOW_STATE_LABEL
        )
        self.assertEqual(
            WorkflowState.objects.all()[0].completion, 0
        )

    def test_delete_workflow_state_no_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        response = self._request_test_workflow_state_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowState.objects.count(), 2)

    def test_delete_workflow_state_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_state_delete_view()
        self.assertEqual(response.status_code, 302)
        self.assertEqual(WorkflowState.objects.count(), 1)

    def test_edit_workflow_state_no_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        workflow_state_label = self.test_workflow_state_1.label

        response = self._request_test_workflow_state_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow_state_1.refresh_from_db()
        self.assertEqual(
            self.test_workflow_state_1.label, workflow_state_label
        )

    def test_edit_workflow_state_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        workflow_state_label = self.test_workflow_state_1.label

        response = self._request_test_workflow_state_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow_state_1.refresh_from_db()
        self.assertNotEquals(
            self.test_workflow_state_1.label, workflow_state_label
        )

    def test_workflow_state_list_no_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        response = self._request_test_workflow_state_list_view()
        self.assertEqual(response.status_code, 404)

    def test_workflow_state_list_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_state_list_view()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, text=self.test_workflow_state_1.label)



from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_acl_delete, icon_acl_list, icon_acl_new, icon_acl_permissions
)
from .permissions import permission_acl_view, permission_acl_edit


def get_kwargs_factory(variable_name):
    def get_kwargs(context):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        content_type = ContentType.objects.get_for_model(
            context[variable_name]
        )
        return {
            'app_label': '"{}"'.format(content_type.app_label),
            'model_name': '"{}"'.format(content_type.model),
            'object_id': '{}.pk'.format(variable_name)
        }

    return get_kwargs


link_acl_create = Link(
    icon_class=icon_acl_new, kwargs=get_kwargs_factory('resolved_object'),
    permissions=(permission_acl_edit,), text=_('New ACL'),
    view='acls:acl_create'
)
link_acl_delete = Link(
    args='resolved_object.pk', icon_class=icon_acl_delete,
    permissions=(permission_acl_edit,), tags='dangerous', text=_('Delete'),
    view='acls:acl_delete'
)
link_acl_list = Link(
    icon_class=icon_acl_list, kwargs=get_kwargs_factory('resolved_object'),
    permissions=(permission_acl_view,), text=_('ACLs'), view='acls:acl_list'
)
link_acl_permissions = Link(
    args='resolved_object.pk', icon_class=icon_acl_permissions,
    permissions=(permission_acl_edit,),
    text=_('Permissions'), view='acls:acl_permissions'
)



from django.db import models, migrations


class Migration(migrations.Migration):
    dependencies = [
        ('document_comments', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='comment',
            name='submit_date',
            field=models.DateTimeField(
                auto_now_add=True, verbose_name='Date time submitted',
                db_index=True
            ),
            preserve_default=True,
        ),
    ]



from django.apps import apps
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models


class ErrorLogEntryManager(models.Manager):
    def register(self, model):
        ErrorLogEntry = apps.get_model(
            app_label='common', model_name='ErrorLogEntry'
        )
        model.add_to_class(
            name='error_logs', value=GenericRelation(ErrorLogEntry)
        )


class UserLocaleProfileManager(models.Manager):
    def get_by_natural_key(self, user_natural_key):
        User = get_user_model()
        try:
            user = User.objects.get_by_natural_key(user_natural_key)
        except User.DoesNotExist:
            raise self.model.DoesNotExist

        return self.get(user__pk=user.pk)



from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    copyright_text='''
        Copyright 2013 Otto Yiu and other contributors
        http://ottoyiu.com

        Permission is hereby granted, free of charge, to any person obtaining
        a copy of this software and associated documentation files (the
        "Software"), to deal in the Software without restriction, including
        without limitation the rights to use, copy, modify, merge, publish,
        distribute, sublicense, and/or sell copies of the Software, and to
        permit persons to whom the Software is furnished to do so, subject to
        the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
        EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
        NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
        LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
        OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
        WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
    ''', module=__name__, name='django-cors-headers', version_string='==3.2.1'
)
PythonDependency(
    copyright_text='''
        Copyright (c) 2011-2015, Tom Christie All rights reserved.

        Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

        Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

        THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    ''', module=__name__, name='djangorestframework', version_string='==3.7.7'
)
PythonDependency(
    copyright_text='''
        Copyright (c) 2015, Warren Jin <jinwarren@gmail.com>

        Permission to use, copy, modify, and/or distribute this software for any
        purpose with or without fee is hereby granted, provided that the above
        copyright notice and this permission notice appear in all copies.

        THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
        WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
        MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
        ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
        WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
        ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
        OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
    ''', module=__name__, name='djangorestframework-recursive', version_string='==0.1.2'
)
PythonDependency(
    module=__name__, name='drf-yasg', version_string='==1.6.0'
)
PythonDependency(
    module=__name__, name='flex', version_string='==6.14.0'
)
PythonDependency(
    module=__name__, name='swagger-spec-validator', version_string='==2.4.3'
)



import logging

from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.reverse import reverse

from mayan.apps.documents.models.document_models import DocumentType

from .models import StagingFolderSource, WebFormSource

logger = logging.getLogger(name=__name__)


class StagingFolderFileUploadSerializer(serializers.Serializer):
    document_type = serializers.PrimaryKeyRelatedField(
        label=_('Document type'), many=False,
        queryset=DocumentType.objects.all(), read_only=False
    )
    expand = serializers.BooleanField(
        default=False, label=_('Expand compressed files'), help_text=_(
            'Upload a compressed file\'s contained files as individual '
            'documents.'
        )
    )


class StagingFolderFileSerializer(serializers.Serializer):
    filename = serializers.CharField(max_length=255)
    image_url = serializers.SerializerMethodField()
    url = serializers.SerializerMethodField()
    encoded_filename = serializers.CharField(max_length=255)
    upload_url = serializers.SerializerMethodField()

    def get_image_url(self, obj):
        return reverse(
            viewname='rest_api:stagingfolderfile-image',
            kwargs={
                'staging_folder_pk': obj.staging_folder.pk,
                'encoded_filename': obj.encoded_filename
            }, request=self.context.get('request')
        )

    def get_upload_url(self, obj):
        return reverse(
            viewname='rest_api:stagingfolderfile-upload',
            kwargs={
                'staging_folder_pk': obj.staging_folder.pk,
                'encoded_filename': obj.encoded_filename,
            }, request=self.context.get('request')
        )

    def get_url(self, obj):
        return reverse(
            viewname='rest_api:stagingfolderfile-detail',
            kwargs={
                'staging_folder_pk': obj.staging_folder.pk,
                'encoded_filename': obj.encoded_filename
            }, request=self.context.get('request')
        )


class StagingFolderSerializer(serializers.HyperlinkedModelSerializer):
    files = serializers.SerializerMethodField()

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:stagingfolder-detail'},
        }
        fields = (
            'delete_after_upload', 'enabled', 'files', 'folder_path', 'id',
            'label', 'preview_height', 'preview_width', 'uncompress', 'url'
        )
        model = StagingFolderSource

    def get_files(self, obj):
        try:
            return [
                StagingFolderFileSerializer(entry, context=self.context).data for entry in obj.get_files()
            ]
        except Exception as exception:
            logger.error('unhandled exception: %s', exception)
            return []


class WebFormSourceSerializer(serializers.Serializer):
    class Meta:
        model = WebFormSource


class NewDocumentSerializer(serializers.Serializer):
    source = serializers.IntegerField()
    document_type = serializers.IntegerField(required=False)
    description = serializers.CharField(required=False)
    expand = serializers.BooleanField(default=False)
    file = serializers.FileField()
    filename = serializers.CharField(required=False)
    use_file_name = serializers.BooleanField(default=False)



from django.http import QueryDict
from django.urls import reverse
from django.utils.encoding import force_bytes
from django.utils.six import PY3


class URL(object):
    def __init__(
        self, path=None, query_string=None, query=None, viewname=None
    ):
        if viewname:
            path = reverse(viewname=viewname)
        self._path = path
        self._query_string = query_string
        self._query = query

        kwargs = {'mutable': True}
        if self._query_string:
            kwargs['query_string'] = self._query_string.encode('utf-8')

        self._args = QueryDict(**kwargs)

        if self._query:
            self.args.update(self._query)

    @property
    def args(self):
        return self._args

    def to_string(self):
        if self._args.keys():
            query = '?{}'.format(self._args.urlencode())
        else:
            query = ''

        if self._path:
            path = self._path
        else:
            path = ''

        result = '{}{}'.format(path, query)

        if PY3:
            return result
        else:
            return force_bytes(result)



import importlib
import logging

from mayan.apps.common import storages
from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.smart_settings.tests.mixins import SmartSettingTestMixin
from mayan.apps.storage.classes import DefinedStorage

from ..literals import STORAGE_NAME_COMMON_SHARED_UPLOADED_FILE
from ..settings import setting_shared_storage_arguments


class CommonStorageSettingsTestCase(SmartSettingTestMixin, BaseTestCase):
    def tearDown(self):
        super(CommonStorageSettingsTestCase, self).tearDown()
        importlib.reload(storages)

    def test_setting_documentimagecache_storage_arguments_invalid_value(self):
        self._set_environment_variable(
            name='MAYAN_{}'.format(
                setting_shared_storage_arguments.global_name
            ), value="invalid_value"
        )
        self.test_case_silenced_logger_new_level = logging.FATAL + 10
        self._silence_logger(name='mayan.apps.storage.classes')

        with self.assertRaises(expected_exception=TypeError) as assertion:
            importlib.reload(storages)
            DefinedStorage.get(
                name=STORAGE_NAME_COMMON_SHARED_UPLOADED_FILE
            ).get_storage_instance()
        self.assertTrue('Unable to initialize' in str(assertion.exception))
        self.assertTrue('shared uploaded' in str(assertion.exception))



from django.shortcuts import get_object_or_404

from rest_framework import status
from rest_framework.response import Response

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models import Document
from mayan.apps.rest_api import generics

from .models import DetachedSignature, EmbeddedSignature
from .permissions import (
    permission_document_version_sign_detached,
    permission_document_version_sign_embedded,
    permission_document_version_signature_delete,
    permission_document_version_signature_upload,
    permission_document_version_signature_view
)
from .serializers import (
    DetachedSignatureSerializer, EmbeddedSignatureSerializer,
    SignDetachedSerializer, SignEmbeddedSerializer
)


class APIDocumentSignDetachedView(generics.GenericAPIView):
    """
    post: Sign a document version with a detached signature.
    """
    serializer_class = SignDetachedSerializer

    def get_document(self):
        return get_object_or_404(
            klass=self.get_document_queryset(), pk=self.kwargs['document_id']
        )

    def get_document_queryset(self):
        if self.request.method == 'POST':
            permission = permission_document_version_sign_detached

        return AccessControlList.objects.restrict_queryset(
            permission=permission, queryset=Document.objects.all(),
            user=self.request.user
        )

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document_version_queryset(),
            pk=self.kwargs['document_version_id']
        )

    def get_document_version_queryset(self):
        return self.get_document().versions.all()

    def get_queryset(self):
        return DetachedSignature.objects.filter(
            document_version=self.get_document_version()
        )

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(
            APIDocumentSignDetachedView, self
        ).get_serializer(*args, **kwargs)

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(
            APIDocumentSignDetachedView, self
        ).get_serializer_context()

        if self.kwargs:
            context.update(
                {
                    'document_version': self.get_document_version(),
                }
            )

        return context

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.sign(
            key_id=request.data['key_id'],
            passphrase=request.data['passphrase']
        )
        return Response(status=status.HTTP_200_OK)


class APIDocumentSignEmbeddedView(generics.GenericAPIView):
    """
    post: Sign a document version with an embedded signature.
    """
    serializer_class = SignEmbeddedSerializer

    def get_document(self):
        return get_object_or_404(
            klass=self.get_document_queryset(), pk=self.kwargs['document_id']
        )

    def get_document_queryset(self):
        if self.request.method == 'POST':
            permission = permission_document_version_sign_embedded

        return AccessControlList.objects.restrict_queryset(
            permission=permission, queryset=Document.objects.all(),
            user=self.request.user
        )

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document_version_queryset(),
            pk=self.kwargs['document_version_id']
        )

    def get_document_version_queryset(self):
        return self.get_document().versions.all()

    def get_queryset(self):
        return DetachedSignature.objects.filter(
            document_version=self.get_document_version()
        )

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(
            APIDocumentSignEmbeddedView, self
        ).get_serializer(*args, **kwargs)

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(
            APIDocumentSignEmbeddedView, self
        ).get_serializer_context()

        if self.kwargs:
            context.update(
                {
                    'document_version': self.get_document_version(),
                }
            )

        return context

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.sign(
            key_id=request.data['key_id'],
            passphrase=request.data['passphrase']
        )
        return Response(status=status.HTTP_200_OK)


class APIDocumentDetachedSignatureListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the detached signatures of a document version.
    post: Create an detached signature for a document version.
    """
    serializer_class = DetachedSignatureSerializer

    def get_document(self):
        return get_object_or_404(
            klass=self.get_document_queryset(), pk=self.kwargs['document_id']
        )

    def get_document_queryset(self):
        if self.request.method == 'GET':
            permission = permission_document_version_signature_view
        elif self.request.method == 'POST':
            permission = permission_document_version_signature_upload

        return AccessControlList.objects.restrict_queryset(
            permission=permission, queryset=Document.objects.all(),
            user=self.request.user
        )

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document_version_queryset(),
            pk=self.kwargs['document_version_id']
        )

    def get_document_version_queryset(self):
        return self.get_document().versions.all()

    def get_queryset(self):
        return DetachedSignature.objects.filter(
            document_version=self.get_document_version()
        )

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(
            APIDocumentDetachedSignatureListView, self
        ).get_serializer(*args, **kwargs)

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(
            APIDocumentDetachedSignatureListView, self
        ).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'document_version': self.get_document_version(),
                }
            )

        return context


class APIDocumentDetachedSignatureView(generics.RetrieveDestroyAPIView):
    """
    delete: Delete an detached signature of the selected document.
    get: Returns the details of the selected detached signature.
    """
    lookup_url_kwarg = 'detached_signature_id'
    serializer_class = DetachedSignatureSerializer

    def get_document(self):
        return get_object_or_404(
            klass=self.get_document_queryset(), pk=self.kwargs['document_id']
        )

    def get_document_queryset(self):
        if self.request.method == 'GET':
            permission = permission_document_version_signature_view
        elif self.request.method == 'POST':
            permission = permission_document_version_signature_view
        elif self.request.method == 'DELETE':
            permission = permission_document_version_signature_delete

        return AccessControlList.objects.restrict_queryset(
            permission=permission, queryset=Document.objects.all(),
            user=self.request.user
        )

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document_version_queryset(),
            pk=self.kwargs['document_version_id']
        )

    def get_document_version_queryset(self):
        return self.get_document().versions.all()

    def get_queryset(self):
        return DetachedSignature.objects.filter(
            document_version=self.get_document_version()
        )

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIDocumentDetachedSignatureView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'document': self.get_document(),
                }
            )

        return context


class APIDocumentEmbeddedSignatureListView(generics.ListAPIView):
    """
    get: Returns a list of all the embedded signatures of a document version.
    """
    serializer_class = EmbeddedSignatureSerializer

    def get_document(self):
        return get_object_or_404(
            klass=self.get_document_queryset(), pk=self.kwargs['document_id']
        )

    def get_document_queryset(self):
        if self.request.method == 'GET':
            permission = permission_document_version_signature_view

        return AccessControlList.objects.restrict_queryset(
            permission=permission, queryset=Document.objects.all(),
            user=self.request.user
        )

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document_version_queryset(),
            pk=self.kwargs['document_version_id']
        )

    def get_document_version_queryset(self):
        return self.get_document().versions.all()

    def get_queryset(self):
        return EmbeddedSignature.objects.filter(
            document_version=self.get_document_version()
        )

    def get_serializer(self, *args, **kwargs):
        if not self.request:
            return None

        return super(
            APIDocumentEmbeddedSignatureListView, self
        ).get_serializer(*args, **kwargs)

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(
            APIDocumentEmbeddedSignatureListView, self
        ).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'document_version': self.get_document_version(),
                }
            )

        return context


class APIDocumentEmbeddedSignatureView(generics.RetrieveAPIView):
    """
    get: Returns the details of the selected embedded signature.
    """
    lookup_url_kwarg = 'embedded_signature_id'
    serializer_class = EmbeddedSignatureSerializer

    def get_document(self):
        return get_object_or_404(
            klass=self.get_document_queryset(), pk=self.kwargs['document_id']
        )

    def get_document_queryset(self):
        if self.request.method == 'GET':
            permission = permission_document_version_signature_view
        elif self.request.method == 'POST':
            permission = permission_document_version_signature_view

        return AccessControlList.objects.restrict_queryset(
            permission=permission, queryset=Document.objects.all(),
            user=self.request.user
        )

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document_version_queryset(),
            pk=self.kwargs['document_version_id']
        )

    def get_document_version_queryset(self):
        return self.get_document().versions.all()

    def get_queryset(self):
        return EmbeddedSignature.objects.filter(
            document_version=self.get_document_version()
        )

    def get_serializer_context(self):
        """
        Extra context provided to the serializer class.
        """
        context = super(APIDocumentEmbeddedSignatureView, self).get_serializer_context()
        if self.kwargs:
            context.update(
                {
                    'document': self.get_document(),
                }
            )

        return context



from collections import OrderedDict

from django import forms

from mayan.apps.common.classes import ModelAttribute
from mayan.apps.common.widgets import NamedMultiWidget

from .literals import EMPTY_LABEL


class TemplateWidget(NamedMultiWidget):
    class Media:
        js = ('templating/js/template_widget.js',)

    def __init__(self, attrs=None, **kwargs):
        widgets = OrderedDict()

        widgets['model_attribute'] = forms.widgets.Select(
            attrs={'data-template-fields': 'model_attribute'}
        )
        widgets['template'] = forms.widgets.Textarea(
            attrs={'rows': 5, 'data-template-fields': 'template'}
        )
        super(TemplateWidget, self).__init__(
            widgets=widgets, attrs=attrs, **kwargs
        )

    def get_context(self, name, value, attrs):
        result = super(TemplateWidget, self).get_context(name, value, attrs)
        result['widget']['subwidgets'][0]['attrs']['required'] = False
        return result

    def decompress(self, value):
        attribute_choices = ModelAttribute.get_all_choices_for(
            model=self.attrs['model']
        )
        choices = []

        choices = attribute_choices
        choices.insert(
            0, ('', EMPTY_LABEL)
        )

        self.widgets['model_attribute'].choices = choices
        return {
            'model_attribute': None, 'template': value
        }

    def value_from_datadict(self, querydict, files, name):
        template = querydict.get('{}_template'.format(name))

        return template



from django.shortcuts import get_object_or_404

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models import Document
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.serializers import DocumentSerializer
from mayan.apps.rest_api import generics

from .models import Index, IndexInstanceNode, IndexTemplateNode
from .permissions import (
    permission_document_indexing_create, permission_document_indexing_delete,
    permission_document_indexing_edit, permission_document_indexing_view
)
from .serializers import (
    IndexInstanceNodeSerializer, IndexSerializer, IndexTemplateNodeSerializer
)


class APIIndexListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the defined indexes.
    post: Create a new index.
    """
    mayan_object_permissions = {'GET': (permission_document_indexing_view,)}
    mayan_view_permissions = {'POST': (permission_document_indexing_create,)}
    queryset = Index.objects.all()
    serializer_class = IndexSerializer


class APIIndexView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected index.
    get: Returns the details of the selected index.
    patch: Partially edit an index.
    put: Edit an index.
    """
    mayan_object_permissions = {
        'GET': (permission_document_indexing_view,),
        'PUT': (permission_document_indexing_edit,),
        'PATCH': (permission_document_indexing_edit,),
        'DELETE': (permission_document_indexing_delete,)
    }
    queryset = Index.objects.all()
    serializer_class = IndexSerializer


class APIIndexNodeInstanceDocumentListView(generics.ListAPIView):
    """
    Returns a list of all the documents contained by a particular index node
    instance.
    """
    mayan_object_permissions = {'GET': (permission_document_view,)}
    serializer_class = DocumentSerializer

    def get_queryset(self):
        index_node_instance = get_object_or_404(
            klass=IndexInstanceNode, pk=self.kwargs['pk']
        )
        AccessControlList.objects.check_access(
            obj=index_node_instance.index,
            permissions=(permission_document_indexing_view,),
            user=self.request.user
        )

        return index_node_instance.documents.all()


class APIIndexTemplateListView(generics.ListAPIView):
    """
    get: Returns a list of all the template nodes for the selected index.
    """
    mayan_object_permissions = {'GET': (permission_document_indexing_view,)}
    serializer_class = IndexTemplateNodeSerializer


class APIIndexTemplateView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected index template node.
    get: Returns the details of the selected index template node.
    patch: Partially edit an index template node.
    put: Edit an index template node.
    """
    mayan_object_permissions = {
        'GET': (permission_document_indexing_view,),
        'PUT': (permission_document_indexing_edit,),
        'PATCH': (permission_document_indexing_edit,),
        'DELETE': (permission_document_indexing_edit,)
    }
    queryset = IndexTemplateNode.objects.all()
    serializer_class = IndexTemplateNodeSerializer


class APIDocumentIndexListView(generics.ListAPIView):
    """
    Returns a list of all the indexes to which a document belongs.
    """
    mayan_object_permissions = {'GET': (permission_document_indexing_view,)}
    serializer_class = IndexInstanceNodeSerializer

    def get_queryset(self):
        document = get_object_or_404(klass=Document, pk=self.kwargs['pk'])
        AccessControlList.objects.check_access(
            obj=document, permissions=(permission_document_view,),
            user=self.request.user
        )

        return document.index_instance_nodes.all()



from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('actstream', '0002_remove_action_data'),
        ('events', '0002_eventsubscription'),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'read', models.BooleanField(
                        default=False, verbose_name='Read'
                    )
                ),
                (
                    'action', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='notifications', to='actstream.Action',
                        verbose_name='Action'
                    )
                ),
                (
                    'user', models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='notifications',
                        to=settings.AUTH_USER_MODEL, verbose_name='User'
                    )
                ),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
            },
        ),
    ]



from django.utils.encoding import force_text

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.tests.base import DocumentTestMixin
from mayan.apps.documents.tests.literals import (
    TEST_DOCUMENT_DESCRIPTION, TEST_DOCUMENT_DESCRIPTION_EDITED,
    TEST_DOCUMENT_LABEL_EDITED, TEST_SMALL_DOCUMENT_PATH
)
from mayan.apps.metadata.models import MetadataType, DocumentTypeMetadataType

from ..models import Index, IndexInstanceNode, IndexTemplateNode

from .literals import (
    TEST_INDEX_TEMPLATE_DOCUMENT_DESCRIPTION_EXPRESSION,
    TEST_INDEX_TEMPLATE_DOCUMENT_LABEL_EXPRESSION,
    TEST_INDEX_TEMPLATE_METADATA_EXPRESSION, TEST_METADATA_TYPE_LABEL,
    TEST_METADATA_TYPE_NAME
)
from .mixins import IndexTestMixin


class IndexTestCase(IndexTestMixin, DocumentTestMixin, BaseTestCase):
    def test_document_description_index(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_INDEX_TEMPLATE_DOCUMENT_DESCRIPTION_EXPRESSION,
            link_documents=True
        )

        self.test_document.description = TEST_DOCUMENT_DESCRIPTION
        self.test_document.save()

        self.test_index.rebuild()

        self.assertEqual(
            IndexInstanceNode.objects.last().value, self.test_document.description
        )
        self.test_document.description = TEST_DOCUMENT_DESCRIPTION_EDITED
        self.test_document.save()

        self.assertEqual(
            IndexInstanceNode.objects.last().value, self.test_document.description
        )

    def test_document_label_index(self):
        self._create_test_index()

        self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression=TEST_INDEX_TEMPLATE_DOCUMENT_LABEL_EXPRESSION,
            link_documents=True
        )

        self.test_index.rebuild()

        self.assertEqual(
            IndexInstanceNode.objects.last().value, self.test_document.label
        )
        self.test_document.label = TEST_DOCUMENT_LABEL_EDITED
        self.test_document.save()

        self.assertEqual(
            IndexInstanceNode.objects.last().value, self.test_document.label
        )

    def test_date_based_index(self):
        self._create_test_index()

        level_year = self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression='{{ document.date_added|date:"Y" }}',
            link_documents=False
        )

        self.test_index.node_templates.create(
            parent=level_year,
            expression='{{ document.date_added|date:"m" }}',
            link_documents=True
        )
        # Index the document created by default
        Index.objects.rebuild()

        self.test_document.delete()

        # Uploading a new should not trigger an error
        self._upload_test_document()

        self.assertEqual(
            list(IndexInstanceNode.objects.values_list('value', flat=True)),
            [
                '', force_text(self.test_document.date_added.year),
                '{:02}'.format(self.test_document.date_added.month)
            ]
        )

        self.assertTrue(
            self.test_document in IndexInstanceNode.objects.last().documents.all()
        )

    def test_dual_level_dual_document_index(self):
        """
        Test creation of an index instance with two first levels with different
        values and two second levels with the same value but as separate
        children of each of the first levels. GitLab issue #391
        """
        with open(TEST_SMALL_DOCUMENT_PATH, mode='rb') as file_object:
            self.test_document_2 = self.test_document_type.new_document(
                file_object=file_object
            )

        self._create_test_index()

        # Create simple index template
        root = self.test_index.template_root
        level_1 = self.test_index.node_templates.create(
            parent=root, expression='{{ document.uuid }}',
            link_documents=False
        )

        self.test_index.node_templates.create(
            parent=level_1, expression='{{ document.label }}',
            link_documents=True
        )

        Index.objects.rebuild()

        # Typecast to sets to make sorting irrelevant in the comparison.
        self.assertEqual(
            set(IndexInstanceNode.objects.values_list('value', flat=True)),
            set(
                [
                    '', force_text(self.test_document_2.uuid), self.test_document_2.label,
                    force_text(self.test_document.uuid), self.test_document.label
                ]
            )
        )

    def test_metadata_indexing(self):
        metadata_type = MetadataType.objects.create(
            name=TEST_METADATA_TYPE_NAME, label=TEST_METADATA_TYPE_LABEL
        )
        DocumentTypeMetadataType.objects.create(
            document_type=self.test_document_type, metadata_type=metadata_type
        )

        self._create_test_index()

        # Create simple index template
        root = self.test_index.template_root
        self.test_index.node_templates.create(
            parent=root, expression=TEST_INDEX_TEMPLATE_METADATA_EXPRESSION,
            link_documents=True
        )

        # Add document metadata value to trigger index node instance creation
        self.test_document.metadata.create(
            metadata_type=metadata_type, value='0001'
        )
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0001']
        )

        # Check that document is in instance node
        instance_node = IndexInstanceNode.objects.get(value='0001')
        self.assertQuerysetEqual(
            instance_node.documents.all(), [repr(self.test_document)]
        )

        # Change document metadata value to trigger index node instance update
        document_metadata = self.test_document.metadata.get(
            metadata_type=metadata_type
        )
        document_metadata.value = '0002'
        document_metadata.save()
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0002']
        )

        # Check that document is in new instance node
        instance_node = IndexInstanceNode.objects.get(value='0002')
        self.assertQuerysetEqual(
            instance_node.documents.all(), [repr(self.test_document)]
        )

        # Check node instance is destoyed when no metadata is available
        self.test_document.metadata.get(metadata_type=metadata_type).delete()
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['']
        )

        # Add document metadata value again to trigger index node instance
        # creation
        self.test_document.metadata.create(
            metadata_type=metadata_type, value='0003'
        )
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0003']
        )

        # Check node instance is destroyed when no documents are contained
        self.test_document.delete()

        # Document is in trash, index structure should remain unchanged
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0003']
        )

        # Document deleted, index structure should update
        self.test_document.delete()
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['']
        )

    def test_multi_level_template_with_no_result_parent(self):
        """
        On a two level template if the first level doesn't return a result
        the indexing should stop. GitLab issue #391.
        """
        self._create_test_index()

        level_1 = self.test_index.node_templates.create(
            parent=self.test_index.template_root,
            expression='',
            link_documents=True
        )

        self.test_index.node_templates.create(
            parent=level_1, expression='{{ document.label }}',
            link_documents=True
        )

        Index.objects.rebuild()

    def test_rebuild_all_indexes(self):
        # Add metadata type and connect to document type
        metadata_type = MetadataType.objects.create(name='test', label='test')
        DocumentTypeMetadataType.objects.create(
            document_type=self.test_document_type, metadata_type=metadata_type
        )

        # Add document metadata value
        self.test_document.metadata.create(
            metadata_type=metadata_type, value='0001'
        )

        self._create_test_index()

        # Create simple index template
        root = self.test_index.template_root
        self.test_index.node_templates.create(
            parent=root, expression='{{ document.metadata_value_of.test }}',
            link_documents=True
        )
        self.assertEqual(
            list(
                IndexTemplateNode.objects.values_list('expression', flat=True)
            ), ['', '{{ document.metadata_value_of.test }}']
        )

        # There should be only a root index instances nodes
        self.assertEqual(IndexInstanceNode.objects.count(), 1)
        self.assertEqual(IndexInstanceNode.objects.first().parent, None)

        # Rebuild all indexes
        Index.objects.rebuild()

        # Check that document is in instance node
        instance_node = IndexInstanceNode.objects.get(value='0001')
        self.assertQuerysetEqual(
            instance_node.documents.all(), [repr(self.test_document)]
        )

    def test_method_get_absolute_url(self):
        self._create_test_index()

        self.assertTrue(self.test_index.get_absolute_url())



import warnings

from mayan.apps.common.tests.base import BaseTestCase

from ..caches import IndexFilesystemCache

from .literals import (
    TEST_CACHE_KEY_BAD_CHARACTERS, TEST_DOCUMENT_PK, TEST_KEY_UNICODE,
    TEST_KEY_UNICODE_HASH, TEST_NODE_PK, TEST_PATH,
)


class MockDocument(object):
    pk = TEST_DOCUMENT_PK


class MockNode(object):
    pk = TEST_NODE_PK


class IndexFilesystemCacheTestCase(BaseTestCase):
    def setUp(self):
        super(IndexFilesystemCacheTestCase, self).setUp()
        self.cache = IndexFilesystemCache()
        self.test_document = MockDocument()
        self.node = MockNode()

    def test_set_path_document(self):
        self.cache.set_path(path=TEST_PATH, document=self.test_document)
        self.assertEqual(
            {'document_pk': TEST_DOCUMENT_PK},
            self.cache.get_path(path=TEST_PATH)
        )

    def test_set_path_document_clear_document(self):
        self.cache.set_path(path=TEST_PATH, document=self.test_document)
        self.cache.clear_document(document=self.test_document)

        self.assertEqual(None, self.cache.get_path(path=TEST_PATH))

    def test_set_path_node(self):
        self.cache.set_path(path=TEST_PATH, node=self.node)
        self.assertEqual(
            {'node_pk': TEST_NODE_PK},
            self.cache.get_path(path=TEST_PATH)
        )

    def test_set_path_node_clear_node(self):
        self.cache.set_path(path=TEST_PATH, node=self.node)
        self.cache.clear_node(node=self.node)

        self.assertEqual(None, self.cache.get_path(path=TEST_PATH))

    def test_valid_cache_key_characters(self):
        with warnings.catch_warnings(record=True) as warning_list:
            self.cache.cache.validate_key(TEST_CACHE_KEY_BAD_CHARACTERS)
            self.assertTrue(len(warning_list) == 1)

        with warnings.catch_warnings(record=True) as warning_list:
            self.cache.cache.validate_key(
                self.cache.get_key_hash(key=TEST_CACHE_KEY_BAD_CHARACTERS)
            )
            self.assertTrue(len(warning_list) == 0)

    def test_hash_non_ascii_key(self):
        self.assertEqual(
            self.cache.get_key_hash(key=TEST_KEY_UNICODE),
            TEST_KEY_UNICODE_HASH
        )



from django.apps import apps
from django.contrib.auth import get_user_model
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from mayan.apps.dashboards.classes import DashboardWidgetNumeric

from .icons import icon_group_list, icon_user_list
from .permissions import permission_group_view, permission_user_view


class DashboardWidgetUserTotal(DashboardWidgetNumeric):
    icon_class = icon_user_list
    label = _('Total users')
    link = reverse_lazy(viewname='user_management:user_list')

    def render(self, request):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        self.count = AccessControlList.objects.restrict_queryset(
            permission=permission_user_view, user=request.user,
            queryset=get_user_model().objects.all()
        ).count()
        return super(DashboardWidgetUserTotal, self).render(request)


class DashboardWidgetGroupTotal(DashboardWidgetNumeric):
    icon_class = icon_group_list
    label = _('Total groups')
    link = reverse_lazy(viewname='user_management:group_list')

    def render(self, request):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )
        Group = apps.get_model(
            app_label='auth', model_name='Group'
        )
        self.count = AccessControlList.objects.restrict_queryset(
            permission=permission_group_view, user=request.user,
            queryset=Group.objects.all()
        ).count()
        return super(DashboardWidgetGroupTotal, self).render(request)



import os

from django.template import loader
from django.template.base import Template
from django.template.context import Context
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.html import mark_safe
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.serialization import yaml_dump, yaml_load
from mayan.apps.task_manager.classes import Worker
from mayan.apps.task_manager.settings import (
    setting_celery_broker_url, setting_celery_result_backend
)


class Variable(object):
    def __init__(self, name, default, environment_name):
        self.name = name
        self.default = default
        self.environment_name = environment_name

    def _get_value(self):
        return os.environ.get(self.environment_name, self.default)

    def get_value(self):
        return mark_safe(self._get_value())


class YAMLVariable(Variable):
    def _get_value(self):
        value = os.environ.get(self.environment_name)
        if value:
            value = yaml_load(stream=value)
        else:
            value = self.default

        return yaml_dump(
            data=value, allow_unicode=True, default_flow_style=True, width=999
        ).replace('...\n', '').replace('\n', '')


@python_2_unicode_compatible
class PlatformTemplate(object):
    _registry = {}
    context = {}
    context_defaults = {}
    label = None
    name = None
    settings = None
    template_name = None
    template_string = None
    variables = None

    @classmethod
    def all(cls):
        return cls._registry.values()

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def register(cls, klass):
        cls._registry[klass.name] = klass

    def __str__(self):
        return force_text(self.get_label())

    def get_context(self):
        return self.context

    def get_context_defaults(self):
        return self.context_defaults

    def get_label(self):
        return self.label or self.name

    def get_settings_context(self):
        result = {}
        for setting in self.settings or ():
            if setting.value:
                result[setting.global_name] = setting.value

        return result

    def get_template_name(self):
        return self.template_name or 'platform/{}.tmpl'.format(self.name)

    def get_variables_context(self):
        result = {}
        for variable in self.variables or ():
            result[variable.name] = variable.get_value()

        return result

    def render(self, context_string=None):
        """
        context_string allows the management command to pass context to this
        method as a JSON string
        """
        context = {}

        context.update(self.get_context_defaults())
        context.update(self.get_settings_context())
        context.update(self.get_variables_context())
        # get_context goes last to server as the override
        context.update(self.get_context())

        if context_string:
            context.update(
                yaml_load(stream=context_string)
            )

        if self.template_string:
            template = Template(template_string=self.template_string)
            return template.render(context=Context(dict_=context))
        else:
            return loader.render_to_string(
                template_name=self.get_template_name(),
                context=context
            )


class PlatformTemplateSupervisord(PlatformTemplate):
    label = _('Template for Supervisord.')
    name = 'supervisord'
    settings = (
        setting_celery_broker_url, setting_celery_result_backend
    )
    variables = (
        Variable(
            name='GUNICORN_WORKERS', default=2,
            environment_name='MAYAN_GUNICORN_WORKERS'
        ),
        Variable(
            name='GUNICORN_TIMEOUT', default=120,
            environment_name='MAYAN_GUNICORN_TIMEOUT'
        ),
        Variable(
            name='INSTALLATION_PATH', default='/opt/mayan-edms',
            environment_name='MAYAN_INSTALLATION_PATH'
        ),
        YAMLVariable(
            name='ALLOWED_HOSTS',
            default=['*'],
            environment_name='MAYAN_ALLOWED_HOSTS'
        ),
        YAMLVariable(
            name='CELERY_BROKER_URL',
            default='redis://:mayanredispassword@127.0.0.1:6379/0',
            environment_name='MAYAN_CELERY_BROKER_URL'
        ),
        YAMLVariable(
            name='CELERY_RESULT_BACKEND',
            default='redis://:mayanredispassword@127.0.0.1:6379/1',
            environment_name='MAYAN_CELERY_RESULT_BACKEND'
        ),
        YAMLVariable(
            name='DATABASES',
            default={
                'default': {
                    'ENGINE': 'django.db.backends.postgresql',
                    'NAME': 'mayan', 'PASSWORD': 'mayanuserpass',
                    'USER': 'mayan', 'HOST': '127.0.0.1'
                }
            },
            environment_name='MAYAN_DATABASES'
        ),
        YAMLVariable
        (
            name='MEDIA_ROOT', default='/opt/mayan-edms/media',
            environment_name='MAYAN_MEDIA_ROOT'
        ),
    )

    def get_context(self):
        return {'workers': Worker.all()}


class PlatformTemplateSupervisordDocker(PlatformTemplate):
    label = _('Template for Supervisord inside a Docker image.')
    name = 'supervisord_docker'

    def get_context(self):
        return {'workers': Worker.all()}


class PlatformTemplateWorkerQueues(PlatformTemplate):
    label = _('Template showing the queues of a worker.')
    name = 'worker_queues'

    variables = (
        Variable(
            name='WORKER_NAME', default=None,
            environment_name='MAYAN_WORKER_NAME'
        ),
    )

    def get_context(self):
        worker_name = self.get_variables_context().get('WORKER_NAME')
        queues = Worker.get(name=worker_name).queues

        return {
            'queues': queues, 'queue_names': sorted(
                map(lambda x: x.name, queues)
            )
        }


PlatformTemplate.register(klass=PlatformTemplateSupervisord)
PlatformTemplate.register(klass=PlatformTemplateSupervisordDocker)
PlatformTemplate.register(klass=PlatformTemplateWorkerQueues)



from rest_framework import serializers

from .models import DocumentPageOCRContent, DocumentTypeSettings


class DocumentPageOCRContentSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('content',)
        model = DocumentPageOCRContent


class DocumentTypeOCRSettingsSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('auto_ocr',)
        model = DocumentTypeSettings



from mayan.apps.appearance.classes import Icon

icon_statistics = Icon(driver_name='fontawesome', symbol='sort-numeric-up')
icon_execute = Icon(driver_name='fontawesome', symbol='cog')
icon_namespace_details = Icon(driver_name='fontawesome', symbol='sort-numeric-up')
icon_namespace_list = Icon(driver_name='fontawesome', symbol='sort-numeric-up')
icon_view = Icon(driver_name='fontawesome', symbol='eye')



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('checkouts', '0006_newversionblock'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documentcheckout',
            options={
                'ordering': ('pk',), 'verbose_name': 'Document checkout',
                'verbose_name_plural': 'Document checkouts'
            },
        ),
    ]



from mayan.apps.document_states.tests.mixins import WorkflowTestMixin
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.document_states.literals import WORKFLOW_ACTION_ON_ENTRY

from ..models.document_models import TrashedDocument
from ..workflow_actions import TrashDocumentAction

from .literals import TEST_TRASH_DOCUMENT_WORKFLOW_ACTION_DOTTED_PATH


class TrashDocumentActionActionTestCase(
    WorkflowTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def test_trash_document_action(self):
        trashed_document_count = TrashedDocument.objects.count()

        self._upload_test_document()

        action = TrashDocumentAction()

        action.execute(context={'document': self.test_document})

        self.assertEqual(
            TrashedDocument.objects.count(), trashed_document_count + 1
        )

    def test_trash_document_action_workflow_execution(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()
        self.test_workflow_states[1].actions.create(
            action_path=TEST_TRASH_DOCUMENT_WORKFLOW_ACTION_DOTTED_PATH,
            label='', when=WORKFLOW_ACTION_ON_ENTRY,
        )
        self.test_workflow.document_types.add(self.test_document_type)

        trashed_document_count = TrashedDocument.objects.count()

        self._upload_test_document()

        self.test_document.workflows.first().do_transition(
            transition=self.test_workflow_transition
        )

        self.assertEqual(
            TrashedDocument.objects.count(), trashed_document_count + 1
        )



from django.apps import apps

from mayan.celery import app


@app.task(ignore_result=True)
def task_send_document(body, sender, subject, recipient, user_mailer_id, as_attachment=False, document_id=None):
    Document = apps.get_model(
        app_label='documents', model_name='Document'
    )
    UserMailer = apps.get_model(
        app_label='mailer', model_name='UserMailer'
    )

    if document_id:
        document = Document.objects.get(pk=document_id)
    else:
        document = None

    user_mailer = UserMailer.objects.get(pk=user_mailer_id)

    user_mailer.send_document(
        as_attachment=as_attachment, body=body, document=document,
        subject=subject, to=recipient
    )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.dependencies.classes import (
    BinaryDependency, JavaScriptDependency, PythonDependency
)

from .settings import setting_scanimage_path

BinaryDependency(
    label='SANE scanimage', help_text=_(
        'Utility provided by the SANE package. Used to control the scanner '
        'and obtained the scanned document image.'
    ), module=__name__, name='scanimage', path=setting_scanimage_path.value
)
JavaScriptDependency(
    module=__name__, name='dropzone', version_string='=5.4.0'
)
PythonDependency(
    module=__name__, name='flanker', version_string='==0.9.11'
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.navigation.classes import Link

from .icons import (
    icon_comment_add, icon_comment_delete, icon_comment_edit,
    icon_comments_for_document
)
from .permissions import (
    permission_document_comment_create, permission_document_comment_delete,
    permission_document_comment_edit, permission_document_comment_view
)

link_comment_add = Link(
    args='object.pk', icon_class=icon_comment_add,
    permissions=(permission_document_comment_create,), text=_('Add comment'),
    view='comments:comment_add',
)
link_comment_delete = Link(
    args='object.pk', icon_class=icon_comment_delete,
    permissions=(permission_document_comment_delete,), tags='dangerous',
    text=_('Delete'), view='comments:comment_delete',
)
link_comment_edit = Link(
    args='object.pk', icon_class=icon_comment_edit,
    permissions=(permission_document_comment_edit,),
    text=_('Edit'), view='comments:comment_edit',
)
link_comments_for_document = Link(
    args='resolved_object.pk', icon_class=icon_comments_for_document,
    permissions=(permission_document_comment_view,), text=_('Comments'),
    view='comments:comments_for_document',
)



from django.db import migrations, models


def operation_make_labels_unique(apps, schema_editor):
    Source = apps.get_model(app_label='sources', model_name='Source')

    for source in Source.objects.using(schema_editor.connection.alias).all():
        # Look for sources with the same label
        duplicate_queryset = Source.objects.using(
            schema_editor.connection.alias
        ).filter(label=source.label).exclude(pk=source.pk)
        if duplicate_queryset:
            # If a duplicate is found, append the id to the original source
            # label
            source.label = '{}__{}'.format(source.label, source.pk)
            source.save()


def operation_make_labels_unique_reverse(apps, schema_editor):
    Source = apps.get_model(app_label='sources', model_name='Source')

    for source in Source.objects.using(schema_editor.connection.alias).all():
        if source.label.endswith('__{}'.format(source.pk)):
            source.label = source.label.replace('__{}'.format(source.pk), '')
            source.save()


class Migration(migrations.Migration):
    dependencies = [
        ('sources', '0018_auto_20180608_0057'),
    ]

    operations = [
        migrations.RunPython(
            code=operation_make_labels_unique,
            reverse_code=operation_make_labels_unique_reverse
        ),
        migrations.AlterField(
            model_name='source',
            name='label',
            field=models.CharField(
                db_index=True, max_length=64, unique=True, verbose_name='Label'
            ),
        ),
    ]



import redis

from django.utils.encoding import force_text

from mayan.apps.dependencies.exceptions import DependenciesException

from ..exceptions import LockError
from ..settings import setting_backend_arguments

from .base import LockingBackend
from .literals import REDIS_LOCK_VERSION_REQUIRED


class RedisLock(LockingBackend):
    @classmethod
    def acquire_lock(cls, name, timeout=None):
        super(RedisLock, cls).acquire_lock(name=name, timeout=timeout)
        return RedisLock(name=name, timeout=timeout)

    @classmethod
    def get_redis_connection(cls):
        redis_url = setting_backend_arguments.value.get('redis_url', None)
        server = redis.from_url(url=redis_url)
        # Force to initialize the connection
        server.client()
        return server

    @classmethod
    def purge_locks(cls):
        super(RedisLock, cls).purge_locks()

    def __init__(self, name, timeout):
        if redis.VERSION < REDIS_LOCK_VERSION_REQUIRED:
            raise DependenciesException(
                'The Redis lock backend requires the Redis Python client '
                'version {} or later.'.format(
                    '.'.join(map(force_text, REDIS_LOCK_VERSION_REQUIRED))
                )
            )

        self.name = name
        redis_lock_instance = self.get_redis_connection().lock(
            name=name, timeout=timeout, sleep=0.1, blocking_timeout=0.1
        )
        if redis_lock_instance.acquire():
            self.redis_lock_instance = redis_lock_instance
        else:
            raise LockError

    def release(self):
        super(RedisLock, self).release()
        try:
            self.redis_lock_instance.release()
        except redis.exceptions.LockNotOwnedError:
            return



from actstream.models import Action

from mayan.apps.common.tests.base import GenericViewTestCase

from ..events import event_quota_created, event_quota_edited
from ..permissions import permission_quota_create, permission_quota_edit

from .mixins import QuotaTestMixin, QuotaViewTestMixin


class QuotaEventsTestCase(QuotaTestMixin, QuotaViewTestMixin, GenericViewTestCase):
    def test_quota_created_event_no_permissions(self):
        Action.objects.all().delete()

        response = self._request_test_quota_create_view()
        self.assertEqual(response.status_code, 403)
        self.assertEqual(Action.objects.count(), 0)

    def test_quota_created_event_with_permissions(self):
        Action.objects.all().delete()

        self.grant_permission(permission=permission_quota_create)
        response = self._request_test_quota_create_view()
        self.assertEqual(response.status_code, 302)

        event = Action.objects.first()

        self.assertEqual(event.verb, event_quota_created.id)
        self.assertEqual(event.target, self.test_quota)
        self.assertEqual(event.actor, self._test_case_user)

    def test_quota_edited_event_no_permissions(self):
        self._create_test_quota()
        Action.objects.all().delete()

        response = self._request_test_quota_edit_view()
        self.assertEqual(response.status_code, 404)
        self.assertEqual(Action.objects.count(), 0)

    def test_quota_edited_event_with_access(self):
        self._create_test_quota()
        Action.objects.all().delete()

        self.grant_access(
            obj=self.test_quota, permission=permission_quota_edit
        )

        response = self._request_test_quota_edit_view()
        self.assertEqual(response.status_code, 302)
        event = Action.objects.first()

        self.assertEqual(event.verb, event_quota_edited.id)
        self.assertEqual(event.target, self.test_quota)
        self.assertEqual(event.actor, self._test_case_user)



from django.core import management

from ...runtime import locking_backend


class Command(management.BaseCommand):
    help = 'Erase all locks (acquired and stale).'

    def handle(self, *args, **options):
        locking_backend.purge_locks()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Common'), name='common')

permission_error_log_view = namespace.add_permission(
    label=_('View error log'), name='error_log_view'
)



from ..permissions import permission_document_view

from .base import GenericDocumentViewTestCase


class DuplicatedDocumentsViewsTestMixin(object):
    def _request_document_duplicates_list_view(self):
        return self.get(
            viewname='documents:document_duplicates_list', kwargs={
                'document_id': self.test_documents[0].pk
            }
        )

    def _request_duplicated_document_list_view(self):
        return self.get(viewname='documents:duplicated_document_list')

    def _upload_duplicate_document(self):
        self._upload_test_document()


class DuplicatedDocumentsViewsTestCase(
    DuplicatedDocumentsViewsTestMixin, GenericDocumentViewTestCase
):
    def test_document_duplicates_list_no_permissions(self):
        self._upload_duplicate_document()

        response = self._request_document_duplicates_list_view()
        self.assertEqual(response.status_code, 404)

    def test_document_duplicates_list_with_access(self):
        self._upload_duplicate_document()
        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_view
        )

        response = self._request_document_duplicates_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self.test_documents[0].label
        )

    def test_document_trashed_duplicates_list_with_full_access(self):
        self._upload_duplicate_document()
        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_view
        )
        self.test_documents[1].delete()

        response = self._request_document_duplicates_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self.test_documents[0].pk
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self.test_documents[1].pk
        )

    def test_duplicated_document_list_no_permissions(self):
        self._upload_duplicate_document()

        response = self._request_duplicated_document_list_view()
        self.assertNotContains(
            response=response, status_code=200,
            text=self.test_documents[0].label
        )

    def test_duplicated_document_list_with_access(self):
        self._upload_duplicate_document()
        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_view
        )

        response = self._request_duplicated_document_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self.test_documents[0].label
        )

    def test_duplicated_trashed_document_list_with_access(self):
        self._upload_duplicate_document()
        self.grant_access(
            obj=self.test_documents[0],
            permission=permission_document_view
        )
        self.grant_access(
            obj=self.test_documents[1],
            permission=permission_document_view
        )
        self.test_documents[1].delete()

        response = self._request_duplicated_document_list_view()
        self.assertNotContains(
            response=response, status_code=200,
            text=self.test_documents[0].pk
        )
        self.assertNotContains(
            response=response, status_code=200,
            text=self.test_documents[1].pk
        )



from .literals import TEST_RECEIVE_KEY, TEST_SEARCH_FINGERPRINT

MOCK_SEARCH_KEYS_RESPONSE = [
    {
        'algo': u'1',
        'date': u'1311475606',
        'expires': u'1643601600',
        'keyid': u'607138F1AECC5A5CA31CB7715F3F7F75D210724D',
        'length': u'2048',
        'type': u'pub',
        'uids': [u'Roberto Rosario <roberto.rosario.gonzalez@gmail.com>']
    }
]


def mock_recv_keys(self, keyserver, *keyids):
    class ImportResult(object):
        count = 1
        fingerprints = [TEST_SEARCH_FINGERPRINT]

    self.import_keys(TEST_RECEIVE_KEY)

    return ImportResult()



class UserPasswordViewTestMixin(object):
    def _request_test_user_password_set_view(self, password):
        return self.post(
            viewname='authentication:user_set_password', kwargs={
                'user_id': self.test_user.pk
            }, data={
                'new_password1': password, 'new_password2': password
            }
        )

    def _request_test_user_password_set_multiple_view(self, password):
        return self.post(
            viewname='authentication:user_multiple_set_password', data={
                'id_list': self.test_user.pk,
                'new_password1': password,
                'new_password2': password
            }
        )



from mayan.apps.appearance.classes import Icon

icon_about = Icon(driver_name='fontawesome', symbol='info')
icon_add_all = Icon(
    driver_name='fontawesome-layers', data=[
        {'class': 'far fa-circle'},
        {'class': 'fas fa-plus', 'transform': 'shrink-6'}
    ]
)
icon_assign_remove_add = Icon(driver_name='fontawesome', symbol='plus')
icon_assign_remove_remove = Icon(driver_name='fontawesome', symbol='minus')
icon_book = Icon(driver_name='fontawesome', symbol='book')
icon_confirm_form_submit = Icon(driver_name='fontawesome', symbol='check')
icon_confirm_form_cancel = Icon(driver_name='fontawesome', symbol='times')
icon_current_user_locale_profile_details = Icon(
    driver_name='fontawesome', symbol='globe'
)
icon_current_user_locale_profile_edit = Icon(
    driver_name='fontawesome', symbol='globe'
)
icon_documentation = Icon(driver_name='fontawesome', symbol='book')
icon_fail = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_forum = Icon(
    driver_name='fontawesome', symbol='life-ring'
)
icon_list_mode_list = Icon(
    driver_name='fontawesome', symbol='list'
)
icon_list_mode_items = Icon(
    driver_name='fontawesome', symbol='th-large'
)
icon_license = Icon(
    driver_name='fontawesome', symbol='certificate'
)
icon_menu_about = Icon(
    driver_name='fontawesome', symbol='info'
)
icon_menu_user = Icon(
    driver_name='fontawesome', symbol='user-circle'
)
icon_object_errors = Icon(
    driver_name='fontawesome', symbol='exclamation-triangle'
)
icon_object_error_list = Icon(
    driver_name='fontawesome', symbol='exclamation-triangle'
)
icon_object_error_list_clear = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_ok = Icon(
    driver_name='fontawesome', symbol='check'
)
icon_remove_all = Icon(
    driver_name='fontawesome-layers', data=[
        {'class': 'far fa-circle'},
        {'class': 'fas fa-minus', 'transform': 'shrink-6'}
    ]
)
icon_setup = Icon(
    driver_name='fontawesome', symbol='cog'
)
icon_social_paypal = Icon(
    driver_name='fontawesomecss', css_classes='fab fa-paypal'
)
icon_sort_down = Icon(driver_name='fontawesome', symbol='sort-down')
icon_sort_up = Icon(driver_name='fontawesome', symbol='sort-up')
icon_source_code = Icon(driver_name='fontawesome', symbol='code-branch')
icon_store = Icon(driver_name='fontawesome', symbol='tshirt')
icon_support = Icon(
    driver_name='fontawesome', symbol='phone'
)
icon_tools = Icon(
    driver_name='fontawesome', symbol='wrench'
)



"""
This module should be called settings.py but is named conf.py to avoid a
clash with the mayan/settings/* module
"""
from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

namespace = Namespace(name='mayan', label=_('Mayan'))

setting_celery_class = namespace.add_setting(
    help_text=_('The class used to instantiate the main Celery app.'),
    global_name='MAYAN_CELERY_CLASS',
    default='celery.Celery'
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.dynamic_search.classes import SearchModel

from .permissions import permission_metadata_type_view

metadata_type_search = SearchModel(
    app_label='metadata', model_name='MetadataType',
    permission=permission_metadata_type_view,
    serializer_path='mayan.apps.metadata.serializers.MetadataTypeSerializer'
)

metadata_type_search.add_model_field(
    field='name', label=_('Name')
)
metadata_type_search.add_model_field(
    field='label', label=_('Label')
)
metadata_type_search.add_model_field(
    field='default', label=_('Default')
)
metadata_type_search.add_model_field(
    field='lookup', label=_('Lookup')
)
metadata_type_search.add_model_field(
    field='validation', label=_('Validator')
)
metadata_type_search.add_model_field(
    field='parser', label=_('Parser')
)



import json

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.events import event_document_properties_edit
from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.events.classes import EventType

from .literals import (
    TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_DOTTED_PATH,
    TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_TEXT_LABEL,
    TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_TEXT_DESCRIPTION,
    TEST_WORKFLOW_STATE_ACTION_LABEL,
    TEST_WORKFLOW_STATE_ACTION_LABEL_2
)
from .mixins import WorkflowStateActionTestMixin, WorkflowTestMixin


class WorkflowInstanceModelTestCase(
    WorkflowTestMixin, GenericDocumentTestCase
):
    def test_workflow_transition_no_condition(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )

        self.assertEqual(
            self.test_workflow_instance.get_transition_choices().count(), 1
        )

    def test_workflow_transition_false_condition(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )

        self.test_workflow_transition.condition = '{{ invalid_variable }}'
        self.test_workflow_transition.save()

        self.assertEqual(
            self.test_workflow_instance.get_transition_choices().count(), 0
        )

    def test_workflow_transition_true_condition(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )

        self.test_workflow_transition.condition = '{{ workflow_instance }}'
        self.test_workflow_transition.save()

        self.assertEqual(
            self.test_workflow_instance.get_transition_choices().count(), 1
        )

    def test_workflow_method_get_absolute_url(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )

        self.test_workflow_instance.get_absolute_url()


class WorkflowModelTestCase(WorkflowTestMixin, BaseTestCase):
    def test_workflow_template_preview(self):
        self._create_test_workflow()
        self.assertTrue(self.test_workflow.get_api_image_url())


class WorkflowStateActionModelTestCase(
    WorkflowStateActionTestMixin, WorkflowTestMixin, GenericDocumentTestCase
):
    def setUp(self):
        super(WorkflowStateActionModelTestCase, self).setUp()
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

    def _get_test_workflow_state_action_execute_flag(self):
        return getattr(
            self.test_workflow_instance,
            '_workflow_state_action_executed', False
        )

    def test_workflow_initial_state_action_no_condition(self):
        self._create_test_workflow_state_action()
        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )
        self.assertTrue(self._get_test_workflow_state_action_execute_flag())

    def test_workflow_initial_state_action_false_condition(self):
        self._create_test_workflow_state_action()
        self.test_workflow_state_action.condition = '{{ invalid_variable }}'
        self.test_workflow_state_action.save()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )
        self.assertFalse(self._get_test_workflow_state_action_execute_flag())

    def test_workflow_initial_state_action_true_condition(self):
        self._create_test_workflow_state_action()
        self.test_workflow_state_action.condition = '{{ workflow_instance }}'
        self.test_workflow_state_action.save()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )
        self.assertTrue(self._get_test_workflow_state_action_execute_flag())

    def test_workflow_state_action_no_condition(self):
        self._create_test_workflow_state_action(workflow_state_index=1)
        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )
        self.test_workflow_instance.do_transition(transition=self.test_workflow_transition)
        self.assertTrue(self._get_test_workflow_state_action_execute_flag())

    def test_workflow_state_action_false_condition(self):
        self._create_test_workflow_state_action(workflow_state_index=1)
        self.test_workflow_state_action.condition = '{{ invalid_variable }}'
        self.test_workflow_state_action.save()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )
        self.test_workflow_instance.do_transition(transition=self.test_workflow_transition)
        self.assertFalse(self._get_test_workflow_state_action_execute_flag())

    def test_workflow_state_action_true_condition(self):
        self._create_test_workflow_state_action(workflow_state_index=1)
        self.test_workflow_state_action.condition = '{{ workflow_instance }}'
        self.test_workflow_state_action.save()

        self.test_workflow_instance = self.test_workflow.launch_for(
            document=self.test_document
        )
        self.test_workflow_instance.do_transition(transition=self.test_workflow_transition)
        self.assertTrue(self._get_test_workflow_state_action_execute_flag())

    def test_workflow_state_action_event_trigger(self):
        # actions 1 and 2 both trigger the transition event, to make this
        # test case independent of the order of execution of actions 1 and 2
        state_1_action_data = json.dumps(obj={
            'document_label': TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_TEXT_LABEL
        })
        self.test_workflow_state_1.actions.create(
            label=TEST_WORKFLOW_STATE_ACTION_LABEL,
            action_path=TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_DOTTED_PATH,
            action_data=state_1_action_data
        )
        self.test_workflow_state_1.actions.create(
            label=TEST_WORKFLOW_STATE_ACTION_LABEL_2,
            action_path=TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_DOTTED_PATH,
            action_data=state_1_action_data
        )

        state_2_action_data = json.dumps(obj={
            'document_description': TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_TEXT_DESCRIPTION
        })
        self.test_workflow_state_2.actions.create(
            label=TEST_WORKFLOW_STATE_ACTION_LABEL,
            action_path=TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_DOTTED_PATH,
            action_data=state_2_action_data
        )

        EventType.refresh()

        self.test_workflow_transition.trigger_events.create(
            event_type=event_document_properties_edit.get_stored_event_type()
        )

        self.test_workflow.launch_for(
            document=self.test_document
        )

        self.assertEqual(
            self.test_document.label,
            TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_TEXT_LABEL
        )
        self.assertEqual(
            self.test_document.description,
            TEST_DOCUMENT_EDIT_WORKFLOW_ACTION_TEXT_DESCRIPTION
        )



from django.conf.urls import url

from .api_views import (
    APIAdvancedSearchView, APISearchModelList, APISearchView
)
from .views import (
    AdvancedSearchView, ResultsView, SearchAgainView, SearchView
)

urlpatterns = [
    url(
        regex=r'^again/(?P<search_model_name>[\.\w]+)/$', name='search_again',
        view=SearchAgainView.as_view()
    ),
    url(
        regex=r'^advanced/(?P<search_model_name>[\.\w]+)/$',
        name='search_advanced', view=AdvancedSearchView.as_view()
    ),
    url(
        regex=r'^advanced/$', name='search_advanced',
        view=AdvancedSearchView.as_view()
    ),
    url(
        regex=r'^results/$', name='results', view=ResultsView.as_view()
    ),
    url(
        regex=r'^results/(?P<search_model_name>[\.\w]+)/$', name='results',
        view=ResultsView.as_view()
    ),
    url(
        regex=r'^search/(?P<search_model_name>[\.\w]+)/$', name='search',
        view=SearchView.as_view()
    )
]

api_urls = [
    url(
        regex=r'^search/(?P<search_model>[\.\w]+)/$', name='search-view',
        view=APISearchView.as_view()
    ),
    url(
        regex=r'^search/advanced/(?P<search_model>[\.\w]+)/$',
        name='advanced-search-view', view=APIAdvancedSearchView.as_view()
    ),
    url(
        regex=r'^search_models/$', name='searchmodel-list',
        view=APISearchModelList.as_view()
    )
]



from pathlib import Path
import shutil

from mayan.apps.documents.literals import STORAGE_NAME_DOCUMENT_VERSION

from ..classes import DefinedStorage
from ..utils import mkdtemp


class StorageProcessorTestMixin(object):
    @classmethod
    def setUpClass(cls):
        super(StorageProcessorTestMixin, cls).setUpClass()
        cls.defined_storage = DefinedStorage.get(
            name=STORAGE_NAME_DOCUMENT_VERSION
        )
        cls.document_storage_kwargs = cls.defined_storage.kwargs

    def setUp(self):
        super(StorageProcessorTestMixin, self).setUp()
        self.temporary_directory = mkdtemp()
        self.path_temporary_directory = Path(self.temporary_directory)
        self.path_test_file = self.path_temporary_directory / 'test_file.txt'

    def tearDown(self):
        super(StorageProcessorTestMixin, self).tearDown()
        shutil.rmtree(self.temporary_directory, ignore_errors=True)
        self.defined_storage.kwargs = self.document_storage_kwargs



from django.conf.urls import url

from .api_views import (
    APIDocumentOCRView, APIDocumentPageOCRContentView,
    APIDocumentTypeOCRSettingsView, APIDocumentVersionOCRView
)
from .views import (
    DocumentOCRContentDeleteView, DocumentOCRContentView,
    DocumentOCRDownloadView,
    DocumentOCRErrorsListView, DocumentPageOCRContentView, DocumentSubmitView,
    DocumentTypeSettingsEditView, DocumentTypeSubmitView, EntryListView
)

urlpatterns = [
    url(
        regex=r'^document_types/submit/$', name='document_type_submit',
        view=DocumentTypeSubmitView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/ocr/settings/$',
        name='document_type_ocr_settings',
        view=DocumentTypeSettingsEditView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/content/$',
        name='document_ocr_content', view=DocumentOCRContentView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/content/delete/$',
        name='document_ocr_content_delete',
        view=DocumentOCRContentDeleteView.as_view()
    ),
    url(
        regex=r'^documents/multiple/content/delete/$',
        name='document_ocr_content_delete_multiple',
        view=DocumentOCRContentDeleteView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/ocr/download/$',
        name='document_ocr_download', view=DocumentOCRDownloadView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/ocr/errors/$',
        name='document_ocr_error_list',
        view=DocumentOCRErrorsListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/submit/$',
        name='document_submit', view=DocumentSubmitView.as_view()
    ),
    url(
        regex=r'^documents/multiple/submit/$',
        name='document_submit_multiple', view=DocumentSubmitView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/content/$',
        name='document_page_ocr_content',
        view=DocumentPageOCRContentView.as_view()
    ),
    url(regex=r'^logs/$', name='entry_list', view=EntryListView.as_view())
]

api_urls = [
    url(
        regex=r'^document_types/(?P<pk>\d+)/ocr/settings/$',
        name='document-type-ocr-settings-view',
        view=APIDocumentTypeOCRSettingsView.as_view()
    ),
    url(
        regex=r'^documents/(?P<pk>\d+)/ocr/submit/$',
        name='document-ocr-submit-view', view=APIDocumentOCRView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/versions/(?P<version_pk>\d+)/ocr/$',
        name='document-version-ocr-submit-view',
        view=APIDocumentVersionOCRView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/versions/(?P<version_pk>\d+)/pages/(?P<page_pk>\d+)/ocr/$',
        name='document-page-ocr-content-view',
        view=APIDocumentPageOCRContentView.as_view()
    ),
]



from datetime import date


class GPGBackend(object):
    def __init__(self, **kwargs):
        self.kwargs = kwargs


class KeyStub(object):
    def __init__(self, raw):
        self.fingerprint = raw['keyid']
        self.key_type = raw['type']
        self.date = date.fromtimestamp(int(raw['date']))
        if raw['expires']:
            self.expires = date.fromtimestamp(int(raw['expires']))
        else:
            self.expires = None
        self.length = raw['length']
        self.user_id = raw['uids']

    @property
    def key_id(self):
        return self.fingerprint[-8:]


class SignatureVerification(object):
    def __init__(self, raw):
        self.user_id = raw['username']
        self.status = raw['status']
        self.key_id = raw['key_id']
        self.pubkey_fingerprint = raw['pubkey_fingerprint']
        self.date = date.fromtimestamp(int(raw['timestamp']))
        if raw['expire_timestamp']:
            self.expires = date.fromtimestamp(int(raw['expire_timestamp']))
        else:
            self.expires = None
        self.trust_text = raw['trust_text']
        self.valid = raw['valid']
        self.stderr = raw['stderr']
        self.fingerprint = raw['fingerprint']
        self.signature_id = raw['signature_id']
        self.trust_level = raw['trust_level']



from actstream.models import Action, any_stream

from ..classes import EventTypeNamespace
from ..models import Notification

from .literals import (
    TEST_EVENT_TYPE_LABEL, TEST_EVENT_TYPE_NAME,
    TEST_EVENT_TYPE_NAMESPACE_LABEL, TEST_EVENT_TYPE_NAMESPACE_NAME
)


class EventListAPIViewTestMixin(object):
    def _request_test_event_list_api_view(self):
        return self.get(viewname='rest_api:event-list')


class EventTestCaseMixin():
    def setUp(self):
        super().setUp()
        Action.objects.all().delete()

    def _get_test_object_event(self, object_name=None):
        test_object = getattr(self, object_name or self._test_event_object_name)

        if test_object:
            return any_stream(obj=test_object).first()
        else:
            return Action.objects.first()


class EventTypeNamespaceAPITestMixin(object):
    def _request_test_event_type_list_api_view(self):
        return self.get(viewname='rest_api:event-type-list')

    def _request_test_event_namespace_list_api_view(self):
        return self.get(viewname='rest_api:event-type-namespace-list')

    def _request_test_event_type_namespace_event_type_list_api_view(self):
        return self.get(
            viewname='rest_api:event-type-namespace-event-type-list',
            kwargs={
                'name': self.test_event_type_namespace.name
            }
        )


class EventTypeTestMixin(object):
    def _create_test_event_type(self):
        self.test_event_type_namespace = EventTypeNamespace(
            label=TEST_EVENT_TYPE_NAMESPACE_LABEL,
            name=TEST_EVENT_TYPE_NAMESPACE_NAME
        )
        self.test_event_type = self.test_event_type_namespace.add_event_type(
            label=TEST_EVENT_TYPE_LABEL,
            name=TEST_EVENT_TYPE_NAME
        )


class NotificationTestMixin(object):
    def _create_test_notification(self):
        self.test_notification = Notification.objects.create(
            user=self._test_case_user, action=Action.objects.first(),
            read=False
        )


class NotificationViewTestMixin(object):
    def _request_test_notification_list_view(self):
        return self.get(viewname='events:user_notifications_list')

    def _request_test_notification_mark_read_all_view(self):
        return self.post(viewname='events:notification_mark_read_all')

    def _request_test_notification_mark_read(self):
        return self.post(
            viewname='events:notification_mark_read', kwargs={
                'notification_id': self.test_notification.pk
            }
        )


class ObjectEventAPITestMixin(object):
    def _request_object_event_list_api_view(self):
        return self.get(
            viewname='rest_api:object-event-list',
            kwargs=self.view_arguments
        )


class UserEventViewsTestMixin(object):
    def _request_test_user_event_type_subscription_list_view(self):
        return self.get(viewname='events:event_types_user_subcriptions_list')



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('metadata', '0012_auto_20190612_0526'),
    ]

    operations = [
        migrations.AlterField(
            model_name='metadatatype',
            name='label',
            field=models.CharField(
                help_text='Short description of this metadata type.',
                max_length=48, verbose_name='Label'
            ),
        ),
    ]



from django.http import Http404
from django.utils.encoding import force_text

from .classes import SearchModel

# Duplicated to keep API compatible until version 4.0
# Merge these two literals and mixins on version 4.0
SEARCH_MODEL_NAME_KWARG = 'search_model_name'
SEARCH_MODEL_NAME_API_KWARG = 'search_model'


class SearchModelMixin(object):
    def get_search_model_name(self):
        return self.kwargs.get(
            SEARCH_MODEL_NAME_KWARG, self.request.GET.get(
                '_{}'.format(SEARCH_MODEL_NAME_KWARG), self.request.POST.get(
                    '_{}'.format(SEARCH_MODEL_NAME_KWARG)
                )
            )
        )

    def get_search_model(self):
        try:
            return SearchModel.get(name=self.get_search_model_name())
        except KeyError as exception:
            raise Http404(force_text(exception))


class SearchModelAPIMixin(SearchModelMixin):
    def get_search_model_name(self):
        return self.kwargs.get(
            SEARCH_MODEL_NAME_API_KWARG, self.request.GET.get(
                '_{}'.format(SEARCH_MODEL_NAME_API_KWARG), self.request.POST.get(
                    '_{}'.format(SEARCH_MODEL_NAME_API_KWARG)
                )
            )
        )



import logging

from django.apps import apps

from .literals import STORAGE_NAME_DOCUMENT_IMAGE

logger = logging.getLogger(name=__name__)


def callback_update_cache_size(setting):
    Cache = apps.get_model(app_label='file_caching', model_name='Cache')
    cache = Cache.objects.get(
        defined_storage_name=STORAGE_NAME_DOCUMENT_IMAGE
    )
    cache.maximum_size = setting.value
    cache.save()



from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured

from mayan.apps.common.tests.mixins import TestModelTestMixin
from mayan.apps.permissions.tests.mixins import (
    PermissionTestMixin, RoleTestCaseMixin, RoleTestMixin
)
from mayan.apps.user_management.tests.mixins import UserTestCaseMixin

from ..classes import ModelPermission
from ..models import AccessControlList
from ..permissions import permission_acl_edit, permission_acl_view


class ACLAPIViewTestMixin(object):
    def _request_test_acl_create_api_view(self, extra_data=None):
        data = {'role_pk': self.test_role.pk}

        if extra_data:
            data.update(extra_data)

        return self.post(
            viewname='rest_api:accesscontrollist-list',
            kwargs=self.test_content_object_view_kwargs, data=data
        )

    def _request_test_acl_delete_api_view(self):
        return self.delete(
            viewname='rest_api:accesscontrollist-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model_name': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }
        )

    def _request_test_acl_detail_api_view(self):
        return self.get(
            viewname='rest_api:accesscontrollist-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model_name': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }
        )

    def _request_test_acl_list_api_view(self):
        return self.get(
            viewname='rest_api:accesscontrollist-list', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model_name': self.test_object_content_type.model,
                'object_id': self.test_object.pk
            }
        )

    def _request_test_acl_permission_delete_api_view(self):
        return self.delete(
            viewname='rest_api:accesscontrollist-permission-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model_name': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk,
                'permission_pk': self.test_permission.stored_permission.pk
            }
        )

    def _request_test_acl_permission_detail_api_view(self):
        return self.get(
            viewname='rest_api:accesscontrollist-permission-detail', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model_name': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk,
                'permission_pk': self.test_acl.permissions.first().pk
            }
        )

    def _request_test_acl_permission_list_api_get_view(self):
        return self.get(
            viewname='rest_api:accesscontrollist-permission-list', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model_name': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }
        )

    def _request_test_acl_permission_list_api_post_view(self):
        return self.post(
            viewname='rest_api:accesscontrollist-permission-list', kwargs={
                'app_label': self.test_object_content_type.app_label,
                'model_name': self.test_object_content_type.model,
                'object_id': self.test_object.pk,
                'pk': self.test_acl.pk
            }, data={'permission_pk': self.test_permission.pk}
        )


class ACLTestCaseMixin(RoleTestCaseMixin, UserTestCaseMixin):
    def setUp(self):
        super(ACLTestCaseMixin, self).setUp()
        if hasattr(self, '_test_case_user'):
            self._test_case_role.groups.add(self._test_case_group)

    def grant_access(self, obj, permission):
        if not hasattr(self, '_test_case_role'):
            raise ImproperlyConfigured(
                'Enable the creation of the test case user, group, and role '
                'in order to enable the usage of ACLs in tests.'
            )

        self._test_case_acl = AccessControlList.objects.grant(
            obj=obj, permission=permission, role=self._test_case_role
        )


class ACLTestMixin(PermissionTestMixin, RoleTestMixin, TestModelTestMixin):
    auto_create_test_role = True
    auto_create_test_object = False

    def setUp(self):
        super(ACLTestMixin, self).setUp()
        if self.auto_create_test_role:
            self._create_test_role()

        if self.auto_create_test_object:
            self._setup_test_object()

    def _create_test_acl(self):
        self.test_acl = AccessControlList.objects.create(
            content_object=self.test_object, role=self.test_role
        )

    def _inject_test_object_content_type(self):
        self.test_object_content_type = ContentType.objects.get_for_model(
            model=self.test_object
        )

        self.test_content_object_view_kwargs = {
            'app_label': self.test_object_content_type.app_label,
            'model_name': self.test_object_content_type.model,
            'object_id': self.test_object.pk
        }

    def _setup_test_object(self, model_name=None, register_model_permissions=True):
        self.TestModel = self._create_test_model(model_name=model_name)
        self.test_object = self.TestModel.objects.create()
        if register_model_permissions:
            ModelPermission.register(
                model=self.TestModel, permissions=(
                    permission_acl_edit, permission_acl_view,
                )
            )

        self._create_test_permission()

        if register_model_permissions:
            ModelPermission.register(
                model=self.TestModel, permissions=(
                    self.test_permission,
                )
            )

        self._inject_test_object_content_type()

    def _setup_test_object_base(self):
        self.test_object_base = self._setup_test_object(
            model_name='TestModelBase'
        )

    def _setup_test_object_proxy(self):
        self.TestModelProxy = self._create_test_model(
            base_class=self.TestModel, model_name='TestModelProxy',
            options={
                'proxy': True
            }
        )
        self.test_object_proxy = self.TestModelProxy.objects.create()


class AccessControlListViewTestMixin(object):
    def _request_test_acl_create_get_view(self):
        return self.get(
            viewname='acls:acl_create',
            kwargs=self.test_content_object_view_kwargs, data={
                'role': self.test_role.pk
            }
        )

    def _request_test_acl_create_post_view(self):
        return self.post(
            viewname='acls:acl_create',
            kwargs=self.test_content_object_view_kwargs, data={
                'role': self.test_role.pk
            }
        )

    def _request_test_acl_delete_view(self):
        return self.post(
            viewname='acls:acl_delete', kwargs={'acl_id': self.test_acl.pk}
        )

    def _request_test_acl_list_view(self):
        return self.get(
            viewname='acls:acl_list',
            kwargs=self.test_content_object_view_kwargs
        )

    def _request_test_acl_permission_list_get_view(self):
        return self.get(
            viewname='acls:acl_permissions', kwargs={
                'acl_id': self.test_acl.pk
            }
        )



from ..permissions import permission_document_view
from ..widgets import DocumentPageThumbnailWidget

from .base import GenericDocumentTestCase, GenericDocumentViewTestCase
from .mixins import DocumentViewTestMixin


class DocumentPageWidgetTestCase(GenericDocumentTestCase):
    def test_document_list_view_document_with_no_pages(self):
        document_thumbnail_widget = DocumentPageThumbnailWidget()
        self.test_document.pages.all().delete()
        result = document_thumbnail_widget.render(instance=self.test_document)

        self.assertTrue(self.test_document.get_absolute_url() in result)


class DocumentPreviewWidgetViewTestCase(
    DocumentViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_preview_page_carousel_widget_render(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_document_preview_view()
        self.assertContains(
            response=response, status_code=200, text='carousel-container'
        )



from django.conf.urls import url

from .api_views import (
    APIDocumentCabinetListView, APICabinetDocumentListView,
    APICabinetDocumentView, APICabinetListView, APICabinetView
)
from .views import (
    DocumentAddToCabinetView, DocumentCabinetListView,
    DocumentRemoveFromCabinetView, CabinetChildAddView, CabinetCreateView,
    CabinetDeleteView, CabinetDetailView, CabinetEditView, CabinetListView,
)

urlpatterns_cabinets = [
    url(
        regex=r'^cabinets/$', name='cabinet_list',
        view=CabinetListView.as_view()
    ),
    url(
        regex=r'^cabinets/create/$', name='cabinet_create',
        view=CabinetCreateView.as_view()
    ),
    url(
        regex=r'^cabinets/(?P<cabinet_id>\d+)/$', name='cabinet_view',
        view=CabinetDetailView.as_view()
    ),
    url(
        regex=r'^cabinets/(?P<cabinet_id>\d+)/children/add/$',
        name='cabinet_child_add', view=CabinetChildAddView.as_view()
    ),
    url(
        regex=r'^cabinets/(?P<cabinet_id>\d+)/delete/$',
        name='cabinet_delete', view=CabinetDeleteView.as_view()
    ),
    url(
        regex=r'^cabinets/(?P<cabinet_id>\d+)/edit/$', name='cabinet_edit',
        view=CabinetEditView.as_view()
    ),
]

urlpatterns_documents_cabinets = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/cabinets/$',
        name='document_cabinet_list', view=DocumentCabinetListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/cabinets/add/$',
        name='document_cabinet_add', view=DocumentAddToCabinetView.as_view()
    ),
    url(
        regex=r'^documents/multiple/cabinets/add/$',
        name='document_multiple_cabinet_add',
        view=DocumentAddToCabinetView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/cabinets/remove/$',
        name='document_cabinet_remove',
        view=DocumentRemoveFromCabinetView.as_view()
    ),
    url(
        regex=r'^documents/multiple/cabinets/remove/$',
        name='multiple_document_cabinet_remove',
        view=DocumentRemoveFromCabinetView.as_view()
    )
]

urlpatterns = []
urlpatterns.extend(urlpatterns_cabinets)
urlpatterns.extend(urlpatterns_documents_cabinets)

api_urls = [
    url(
        regex=r'^cabinets/(?P<pk>[0-9]+)/documents/(?P<document_pk>[0-9]+)/$',
        name='cabinet-document', view=APICabinetDocumentView.as_view()
    ),
    url(
        regex=r'^cabinets/(?P<pk>[0-9]+)/documents/$',
        name='cabinet-document-list',
        view=APICabinetDocumentListView.as_view()
    ),
    url(
        regex=r'^cabinets/(?P<pk>[0-9]+)/$', name='cabinet-detail',
        view=APICabinetView.as_view()
    ),
    url(
        regex=r'^cabinets/$', name='cabinet-list',
        view=APICabinetListView.as_view()
    ),
    url(
        regex=r'^documents/(?P<pk>[0-9]+)/cabinets/$',
        name='document-cabinet-list',
        view=APIDocumentCabinetListView.as_view()
    ),
]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('web_links', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='weblink',
            name='label',
            field=models.CharField(
                db_index=True, help_text='A short text describing the '
                'web link.', max_length=96, verbose_name='Label'
            ),
        ),
        migrations.AlterField(
            model_name='weblink',
            name='template',
            field=models.TextField(
                help_text='Template that will be used to craft the '
                'final URL of the web link. The {{ document }} variable '
                'is available to the template.', verbose_name='Template'
            ),
        ),
    ]



from django.conf.urls import url

from .api_views import (
    APIDocumentPageContentView, APIDocumentTypeParsingSettingsView
)
from .views import (
    DocumentContentView, DocumentContentDeleteView,
    DocumentContentDownloadView, DocumentPageContentView,
    DocumentParsingErrorsListView, DocumentSubmitView,
    DocumentTypeSettingsEditView, DocumentTypeSubmitView, ParseErrorListView
)

urlpatterns = [
    url(
        regex=r'^documents/(?P<document_id>\d+)/content/$',
        name='document_content', view=DocumentContentView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/content/delete/$',
        name='document_content_delete',
        view=DocumentContentDeleteView.as_view()
    ),
    url(
        regex=r'^documents/multiple/content/delete/$',
        name='document_content_delete_multiple',
        view=DocumentContentDeleteView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/content/download/$',
        name='document_content_download',
        view=DocumentContentDownloadView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/submit/$',
        name='document_submit', view=DocumentSubmitView.as_view()
    ),
    url(
        regex=r'^documents/multiple/submit/$',
        name='document_submit_multiple', view=DocumentSubmitView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_id>\d+)/errors/$',
        name='document_parsing_error_list',
        view=DocumentParsingErrorsListView.as_view()
    ),
    url(
        regex=r'^documents/pages/(?P<document_page_id>\d+)/content/$',
        name='document_page_content', view=DocumentPageContentView.as_view()
    ),
    url(
        regex=r'^document_types/submit/$', name='document_type_submit',
        view=DocumentTypeSubmitView.as_view()
    ),
    url(
        regex=r'^document_types/(?P<document_type_id>\d+)/parsing/settings/$',
        name='document_type_parsing_settings',
        view=DocumentTypeSettingsEditView.as_view()
    ),
    url(
        regex=r'^errors/all/$', name='error_list',
        view=ParseErrorListView.as_view()
    )
]

api_urls = [
    url(
        regex=r'^document_types/(?P<pk>\d+)/parsing/settings/$',
        name='document-type-parsing-settings-view',
        view=APIDocumentTypeParsingSettingsView.as_view()
    ),
    url(
        regex=r'^documents/(?P<document_pk>\d+)/versions/(?P<version_pk>\d+)/pages/(?P<page_pk>\d+)/content/$',
        name='document-page-content-view',
        view=APIDocumentPageContentView.as_view()
    )
]



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

from .literals import DEFAULT_GPG_PATH

namespace = Namespace(label=_('Signatures'), name='django_gpg')


setting_gpg_backend = namespace.add_setting(
    global_name='SIGNATURES_BACKEND',
    default='mayan.apps.django_gpg.backends.python_gnupg.PythonGNUPGBackend',
    help_text=_(
        'Full path to the backend to be used to handle keys and signatures.'
    )
)
setting_gpg_backend_arguments = namespace.add_setting(
    global_name='SIGNATURES_BACKEND_ARGUMENTS',
    default={
        'gpg_path': DEFAULT_GPG_PATH,
    }
)
setting_keyserver = namespace.add_setting(
    global_name='SIGNATURES_KEYSERVER', default='pool.sks-keyservers.net',
    help_text=_('Keyserver used to query for keys.')
)



import functools

from django.db import transaction

from .literals import (
    EVENT_MANAGER_ORDER_AFTER, EVENT_MANAGER_ORDER_BEFORE
)


def method_event(event_manager_class, **event_manager_kwargs):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(self, *args, **kwargs):
            event_manager = event_manager_class(
                instance=self, **event_manager_kwargs
            )
            event_manager.prepare()
            event_manager.pop_event_attributes()

            with transaction.atomic():
                if event_manager.order == EVENT_MANAGER_ORDER_BEFORE:
                    event_manager.commit()

                result = func(self, *args, **kwargs)

                if event_manager.order == EVENT_MANAGER_ORDER_AFTER:
                    event_manager.commit()

            return result
        return wrapper
    return decorator



from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('OCR'), name='ocr')

permission_ocr_document = namespace.add_permission(
    label=_('Submit documents for OCR'), name='ocr_document'
)
permission_ocr_content_view = namespace.add_permission(
    label=_('View the transcribed text from document'),
    name='ocr_content_view'
)
permission_document_type_ocr_setup = namespace.add_permission(
    label=_('Change document type OCR settings'),
    name='ocr_document_type_setup'
)



class DocumentContentToolsViewsTestMixin(object):
    def _request_document_parsing_error_list_view(self):
        return self.get(viewname='document_parsing:error_list')

    def _request_document_parsing_tool_view(self):
        return self.post(
            viewname='document_parsing:document_type_submit', data={
                'document_type': self.test_document_type.pk
            }
        )


class DocumentContentViewTestMixin(object):
    def _request_test_document_content_delete_view(self):
        return self.post(
            viewname='document_parsing:document_content_delete', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_content_download_view(self):
        return self.get(
            viewname='document_parsing:document_content_download', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_content_view(self):
        return self.get(
            'document_parsing:document_content', kwargs={
                'document_id': self.test_document.pk
            }
        )

    def _request_test_document_page_content_view(self):
        return self.get(
            viewname='document_parsing:document_page_content', kwargs={
                'document_page_id': self.test_document.pages.first().pk,
            }
        )

    def _request_test_document_parsing_error_list_view(self):
        return self.get(
            viewname='document_parsing:document_parsing_error_list', kwargs={
                'document_id': self.test_document.pk,
            }
        )


class DocumentTypeContentViewsTestMixin(object):
    def _request_test_document_type_parsing_settings(self):
        return self.get(
            viewname='document_parsing:document_type_parsing_settings',
            kwargs={'document_type_id': self.test_document_type.pk}
        )


class DocumentTypeParsingSettingsAPIViewTestMixin():
    def _request_document_type_parsing_settings_details_api_view(self):
        return self.get(
            viewname='rest_api:document-type-parsing-settings-view',
            kwargs={'pk': self.test_document_type.pk}
        )

    def _request_document_type_parsing_settings_patch_api_view(self):
        return self.patch(
            viewname='rest_api:document-type-parsing-settings-view',
            kwargs={'pk': self.test_document_type.pk},
            data={'auto_parsing': True}
        )

    def _request_document_type_parsing_settings_put_api_view(self):
        return self.put(
            viewname='rest_api:document-type-parsing-settings-view',
            kwargs={'pk': self.test_document_type.pk},
            data={'auto_parsing': True}
        )



import os
import shutil

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.tests.literals import TEST_NON_ASCII_DOCUMENT_PATH
from mayan.apps.storage.utils import mkdtemp

from ..classes import StagingFile

from .mocks import MockStagingFolder


class StagingFileTestCase(BaseTestCase):
    def test_unicode_staging_file(self):
        temporary_directory = mkdtemp()
        shutil.copy(TEST_NON_ASCII_DOCUMENT_PATH, temporary_directory)

        filename = os.path.basename(TEST_NON_ASCII_DOCUMENT_PATH)

        staging_file_1 = StagingFile(
            staging_folder=MockStagingFolder(),
            filename=filename
        )

        staging_file_2 = StagingFile(
            staging_folder=MockStagingFolder(),
            encoded_filename=staging_file_1.encoded_filename
        )

        self.assertEqual(filename, staging_file_2.filename)

        shutil.rmtree(temporary_directory)



from mayan.apps.common.tests.base import BaseTestCase, GenericViewTestCase

from .mixins import DocumentTestMixin


class GenericDocumentTestCase(DocumentTestMixin, BaseTestCase):
    """Base test case when testing models or classes"""


class GenericDocumentViewTestCase(DocumentTestMixin, GenericViewTestCase):
    """Base test case when testing views"""



class SearchViewTestMixin(object):
    def _request_search_results_view(self, data, kwargs=None, query=None):
        return self.get(
            viewname='search:results', kwargs=kwargs, data=data, query=query
        )



from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..models import Workflow
from ..permissions import (
    permission_workflow_create, permission_workflow_delete,
    permission_workflow_edit, permission_workflow_view,
    permission_workflow_tools
)

from .literals import TEST_WORKFLOW_LABEL, TEST_WORKFLOW_LABEL_EDITED
from .mixins import (
    WorkflowTestMixin, WorkflowToolViewTestMixin, WorkflowViewTestMixin
)


class WorkflowViewTestCase(
    WorkflowTestMixin, WorkflowViewTestMixin, GenericViewTestCase
):
    def test_workflow_create_view_no_permission(self):
        response = self._request_test_workflow_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Workflow.objects.count(), 0)

    def test_workflow_create_view_with_permission(self):
        self.grant_permission(permission=permission_workflow_create)

        response = self._request_test_workflow_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Workflow.objects.count(), 1)
        self.assertEqual(Workflow.objects.all()[0].label, TEST_WORKFLOW_LABEL)

    def test_workflow_delete_view_no_access(self):
        self._create_test_workflow()

        response = self._request_test_workflow_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(self.test_workflow in Workflow.objects.all())

    def test_workflow_delete_view_with_access(self):
        self._create_test_workflow()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_delete
        )

        response = self._request_test_workflow_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(self.test_workflow in Workflow.objects.all())

    def test_workflow_edit_view_no_access(self):
        self._create_test_workflow()

        response = self._request_test_workflow_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow.refresh_from_db()
        self.assertEqual(self.test_workflow.label, TEST_WORKFLOW_LABEL)

    def test_workflow_edit_view_with_access(self):
        self._create_test_workflow()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow.refresh_from_db()
        self.assertEqual(self.test_workflow.label, TEST_WORKFLOW_LABEL_EDITED)

    def test_workflow_list_view_no_access(self):
        self._create_test_workflow()

        response = self._request_test_workflow_list_view()

        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, text=self.test_workflow.label)

    def test_workflow_list_view_with_access(self):
        self._create_test_workflow()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_list_view()
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, text=self.test_workflow.label)

    def test_workflow_template_preview_view_no_access(self):
        self._create_test_workflow()

        response = self._request_test_workflow_template_preview_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(self.test_workflow in Workflow.objects.all())

    def test_workflow_template_preview_view_with_access(self):
        self._create_test_workflow()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )
        response = self._request_test_workflow_template_preview_view()
        self.assertEqual(response.status_code, 200)


class WorkflowTemplateDocumentViewTestCase(
    WorkflowTestMixin, WorkflowViewTestMixin, GenericDocumentViewTestCase
):
    def test_workflows_launch_view_no_permission(self):
        self._create_test_workflow(add_document_type=True)
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        response = self._request_test_workflow_launch_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(self.test_document.workflows.count(), 0)

    def test_workflows_launch_view_with_permission(self):
        self._create_test_workflow(add_document_type=True)
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_tools
        )

        response = self._request_test_workflow_launch_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_document.workflows.first().workflow, self.test_workflow
        )


class WorkflowToolViewTestCase(
    WorkflowTestMixin, WorkflowToolViewTestMixin, GenericDocumentViewTestCase
):
    def test_tool_launch_workflows_view_no_permission(self):
        self._create_test_workflow(add_document_type=True)
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.assertEqual(self.test_document.workflows.count(), 0)

        response = self._request_workflow_launch_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(self.test_document.workflows.count(), 0)

    def test_tool_launch_workflows_view_with_permission(self):
        self._create_test_workflow(add_document_type=True)
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.grant_permission(permission=permission_workflow_tools)
        self.assertEqual(self.test_document.workflows.count(), 0)

        response = self._request_workflow_launch_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_document.workflows.first().workflow, self.test_workflow
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig


class DashboardsApp(MayanAppConfig):
    app_namespace = 'dashboards'
    app_url = 'dashboards'
    has_rest_api = False
    has_tests = False
    name = 'mayan.apps.dashboards'
    verbose_name = _('Dashboards')



from django.utils.encoding import force_text

from rest_framework import status

from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..models import Cabinet
from ..permissions import (
    permission_cabinet_add_document, permission_cabinet_create,
    permission_cabinet_delete, permission_cabinet_edit,
    permission_cabinet_remove_document, permission_cabinet_view
)

from .mixins import (
    CabinetAPIViewTestMixin, CabinetTestMixin,
    DocumentCabinetAPIViewTestMixin
)


class CabinetAPITestCase(
    CabinetAPIViewTestMixin, CabinetTestMixin, BaseAPITestCase
):
    def test_cabinet_create_api_view_no_permission(self):
        cabinet_count = Cabinet.objects.count()

        response = self._request_test_cabinet_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertEqual(cabinet_count, Cabinet.objects.count())

    def test_cabinet_create_api_view_with_permission(self):
        self.grant_permission(permission=permission_cabinet_create)

        response = self._request_test_cabinet_create_api_view()

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['id'], self.test_cabinet.pk)
        self.assertEqual(response.data['label'], self.test_cabinet.label)

        self.assertEqual(Cabinet.objects.count(), 1)

    def test_cabinet_delete_api_view_no_permssions(self):
        self._create_test_cabinet()

        cabinet_count = Cabinet.objects.count()

        response = self._request_test_cabinet_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(Cabinet.objects.count(), cabinet_count)

    def test_cabinet_delete_api_view_with_access(self):
        self._create_test_cabinet()

        self.grant_access(
            obj=self.test_cabinet, permission=permission_cabinet_delete
        )

        cabinet_count = Cabinet.objects.count()

        response = self._request_test_cabinet_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(Cabinet.objects.count(), cabinet_count - 1)

    def test_cabinet_edit_api_patch_view_no_pemission(self):
        self._create_test_cabinet()

        cabinet_label = self.test_cabinet.label

        response = self._request_test_cabinet_edit_api_patch_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_cabinet.refresh_from_db()
        self.assertEqual(cabinet_label, self.test_cabinet.label)

    def test_cabinet_edit_api_patch_view_with_access(self):
        self._create_test_cabinet()

        self.grant_access(
            obj=self.test_cabinet, permission=permission_cabinet_edit
        )

        cabinet_label = self.test_cabinet.label

        response = self._request_test_cabinet_edit_api_patch_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_cabinet.refresh_from_db()
        self.assertNotEqual(cabinet_label, self.test_cabinet.label)

    def test_cabinet_edit_api_put_view_no_pemission(self):
        self._create_test_cabinet()

        cabinet_label = self.test_cabinet.label

        response = self._request_test_cabinet_edit_api_put_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.test_cabinet.refresh_from_db()
        self.assertEqual(cabinet_label, self.test_cabinet.label)

    def test_cabinet_edit_api_put_view_with_access(self):
        self._create_test_cabinet()

        self.grant_access(
            obj=self.test_cabinet, permission=permission_cabinet_edit
        )

        cabinet_label = self.test_cabinet.label

        response = self._request_test_cabinet_edit_api_put_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.test_cabinet.refresh_from_db()
        self.assertNotEqual(cabinet_label, self.test_cabinet.label)

    def test_cabinet_list_api_view_no_permission(self):
        self._create_test_cabinet()

        response = self._request_test_cabinet_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_cabinet_list_api_view_with_access(self):
        self._create_test_cabinet()

        self.grant_access(
            obj=self.test_cabinet, permission=permission_cabinet_view
        )

        response = self._request_test_cabinet_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['label'], self.test_cabinet.label
        )


class CabinetDocumentAPITestCase(
    CabinetAPIViewTestMixin, CabinetTestMixin, DocumentTestMixin,
    BaseAPITestCase
):
    auto_upload_test_document = False

    def test_cabinet_create_with_single_document(self):
        self._upload_test_document()

        self.grant_permission(permission=permission_cabinet_create)

        response = self._request_test_cabinet_create_api_view(
            extra_data={
                'documents_pk_list': '{}'.format(
                    self.test_document.pk
                )
            }
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(response.data['id'], self.test_cabinet.pk)
        self.assertEqual(response.data['label'], self.test_cabinet.label)

        self.assertQuerysetEqual(
            self.test_cabinet.documents.all(), (repr(self.test_document),)
        )

    def test_cabinet_create_with_multiple_documents(self):
        self._upload_test_document()
        self._upload_test_document()

        documents_pk_list = ','.join(
            [force_text(document.pk) for document in self.test_documents]
        )

        self.grant_permission(permission=permission_cabinet_create)

        response = self._request_test_cabinet_create_api_view(
            extra_data={
                'documents_pk_list': documents_pk_list
            }
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['id'], self.test_cabinet.pk)
        self.assertEqual(response.data['label'], self.test_cabinet.label)

        self.assertEqual(Cabinet.objects.count(), 1)

        self.assertQuerysetEqual(
            qs=self.test_cabinet.documents.all(),
            values=map(repr, self.test_documents)
        )

    def test_cabinet_document_remove_api_view(self):
        self._upload_test_document()

        self._create_test_cabinet()
        self.test_cabinet.documents.add(self.test_document)

        self.grant_permission(
            permission=permission_cabinet_remove_document
        )

        response = self._request_test_cabinet_document_remove_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(self.test_cabinet.documents.count(), 0)

    def test_cabinet_document_detail_api_view(self):
        self._upload_test_document()

        self._create_test_cabinet()

        self.test_cabinet.documents.add(self.test_document)

        self.grant_permission(
            permission=permission_cabinet_view
        )
        self.grant_permission(
            permission=permission_document_view
        )
        response = self.get(
            viewname='rest_api:cabinet-document', kwargs={
                'pk': self.test_cabinet.pk,
                'document_pk': self.test_document.pk
            }
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['uuid'], force_text(self.test_document.uuid)
        )

    def test_cabinet_document_list_api_view(self):
        self._upload_test_document()

        self._create_test_cabinet()

        self.test_cabinet.documents.add(self.test_document)

        self.grant_permission(permission=permission_cabinet_view)
        self.grant_permission(permission=permission_document_view)

        response = self.get(
            viewname='rest_api:cabinet-document-list', kwargs={
                'pk': self.test_cabinet.pk
            }
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['results'][0]['uuid'],
            force_text(self.test_document.uuid)
        )

    def test_cabinet_add_document_api_view(self):
        self._upload_test_document()

        self._create_test_cabinet()

        response = self.post(
            data={
                'documents_pk_list': '{}'.format(self.test_document.pk)
            }, kwargs={
                'pk': self.test_cabinet.pk
            }, viewname='rest_api:cabinet-document-list'
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertQuerysetEqual(
            self.test_cabinet.documents.all(), (repr(self.test_document),)
        )

    def test_cabinet_add_multiple_documents_api_view(self):
        self._upload_test_document()
        self._upload_test_document()

        documents_pk_list = ','.join(
            [force_text(document.pk) for document in self.test_documents]
        )

        self._create_test_cabinet()

        self.grant_permission(permission=permission_cabinet_add_document)
        response = self.post(
            data={
                'documents_pk_list': documents_pk_list
            }, kwargs={
                'pk': self.test_cabinet.pk
            }, viewname='rest_api:cabinet-document-list'
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertQuerysetEqual(
            qs=self.test_cabinet.documents.all(),
            values=map(repr, self.test_documents)
        )


class DocumentCabinetAPITestCase(
    CabinetAPIViewTestMixin, CabinetTestMixin,
    DocumentCabinetAPIViewTestMixin, DocumentTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super(DocumentCabinetAPITestCase, self).setUp()
        self._upload_test_document()
        self._create_test_cabinet()
        self.test_cabinet.documents.add(self.test_document)

    def test_document_cabinet_list_view_no_permission(self):
        response = self._request_test_document_cabinet_list_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertTrue('count' not in response.data)

    def test_document_cabinet_list_view_with_document_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view,
        )

        response = self._request_test_document_cabinet_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

    def test_document_cabinet_list_view_with_cabinet_access(self):
        self.grant_access(
            obj=self.test_cabinet, permission=permission_cabinet_view,
        )

        response = self._request_test_document_cabinet_list_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertTrue('count' not in response.data)

    def test_document_cabinet_list_view_with_full_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view,
        )
        self.grant_access(
            obj=self.test_cabinet, permission=permission_cabinet_view,
        )

        response = self._request_test_document_cabinet_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['count'], Cabinet.objects.all().count()
        )
        self.assertEqual(
            response.data['results'][0]['id'], self.test_cabinet.pk
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(
    label=_('Document signatures'), name='document_signatures'
)

permission_document_version_sign_detached = namespace.add_permission(
    label=_('Sign documents with detached signatures'),
    name='document_version_sign_detached'
)
permission_document_version_sign_embedded = namespace.add_permission(
    label=_('Sign documents with embedded signatures'),
    name='document_version_sign_embedded'
)
permission_document_version_signature_delete = namespace.add_permission(
    label=_('Delete detached signatures'),
    name='document_version_signature_delete'
)
permission_document_version_signature_download = namespace.add_permission(
    label=_('Download detached document signatures'),
    name='document_version_signature_download'
)
permission_document_version_signature_upload = namespace.add_permission(
    label=_('Upload detached document signatures'),
    name='document_version_signature_upload'
)
permission_document_version_signature_verify = namespace.add_permission(
    label=_('Verify document signatures'),
    name='document_version_signature_verify'
)
permission_document_version_signature_view = namespace.add_permission(
    label=_('View details of document signatures'),
    name='document_version_signature_view'
)



from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.reverse import reverse

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_type_models import DocumentType
from mayan.apps.documents.serializers import DocumentTypeSerializer
from mayan.apps.documents.permissions import permission_document_type_edit

from .models import ResolvedWebLink, WebLink


class WebLinkSerializer(serializers.HyperlinkedModelSerializer):
    document_types = DocumentTypeSerializer(read_only=True, many=True)

    class Meta:
        extra_kwargs = {
            'url': {
                'lookup_url_kwarg': 'pk',
                'view_name': 'rest_api:web_link-detail'
            },
        }
        fields = (
            'document_types', 'enabled', 'id', 'label', 'template', 'url'
        )
        model = WebLink


class ResolvedWebLinkSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.SerializerMethodField()
    navigation_url = serializers.SerializerMethodField()

    class Meta:
        fields = ('id', 'navigation_url', 'url')
        model = ResolvedWebLink

    def get_url(self, instance):
        return reverse(
            viewname='rest_api:resolved_web_link-detail',
            kwargs={
                'pk': self.context['document'].pk,
                'resolved_web_link_pk': instance.pk
            }, request=self.context['request'],
            format=self.context['format']
        )

    def get_navigation_url(self, instance):
        return reverse(
            viewname='rest_api:resolved_web_link-navigate',
            kwargs={
                'pk': self.context['document'].pk,
                'resolved_web_link_pk': instance.pk
            }, request=self.context['request'],
            format=self.context['format']
        )


class WritableWebLinkSerializer(serializers.ModelSerializer):
    document_types_pk_list = serializers.CharField(
        help_text=_(
            'Comma separated list of document type primary keys to which '
            'this web link will be attached.'
        ), required=False
    )

    class Meta:
        extra_kwargs = {
            'url': {'view_name': 'rest_api:web_link-detail'},
        }
        fields = (
            'document_types_pk_list', 'enabled', 'label', 'id', 'template',
            'url'
        )
        model = WebLink

    def validate(self, attrs):
        document_types_pk_list = attrs.pop('document_types_pk_list', None)

        if document_types_pk_list:
            queryset = AccessControlList.objects.restrict_queryset(
                permission=permission_document_type_edit,
                queryset=DocumentType.objects.filter(
                    pk__in=document_types_pk_list.split(',')
                ), user=self.context['request'].user
            )

            attrs['document_types'] = queryset
        else:
            attrs['document_types'] = ()
        return attrs



from django.template.loader import render_to_string

from .permissions import permission_tag_view


def widget_document_tags(document, user):
    """
    A tag widget that displays the tags for the given document
    """
    return render_to_string(
        template_name='tags/document_tags_widget.html', context={
            'tags': document.get_tags(
                permission=permission_tag_view, user=user
            )
        }
    )


def widget_single_tag(tag):
    return render_to_string(
        template_name='tags/tag_widget.html', context={'tag': tag}
    )



from django.utils.encoding import force_text

from mayan.apps.common.tests.base import BaseTestCase
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.search import document_search
from mayan.apps.documents.tests.mixins import DocumentTestMixin

from ..backends.django import DjangoSearchBackend


class DjangoSearchBackendDocumentSearchTestCase(
    DocumentTestMixin, BaseTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super(DjangoSearchBackendDocumentSearchTestCase, self).setUp()
        self.search_backend = DjangoSearchBackend()

    def test_simple_search_after_related_name_change(self):
        """
        Test that simple search works after related_name changes to
        document versions and document version pages
        """
        self._upload_test_document(label='first_doc')

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'q': 'first'}, user=self._test_case_user
        )

        self.assertEqual(queryset.count(), 1)
        self.assertTrue(self.test_document in queryset)

    def test_advanced_search_after_related_name_change(self):
        # Test versions__filename
        self._upload_test_document()
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': self.test_document.label},
            user=self._test_case_user
        )

        self.assertEqual(queryset.count(), 1)
        self.assertTrue(self.test_document in queryset)

        # Test versions__mimetype
        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={
                'versions__mimetype': self.test_document.file_mimetype
            }, user=self._test_case_user
        )

        self.assertEqual(queryset.count(), 1)
        self.assertTrue(self.test_document in queryset)

    def test_meta_only(self):
        self._upload_test_document(label='first_doc')
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'q': 'OR first'}, user=self._test_case_user
        )

        self.assertEqual(queryset.count(), 1)

    def test_simple_or_search(self):
        self._upload_test_document(label='first_doc')
        self.grant_access(
            obj=self.test_documents[0], permission=permission_document_view
        )
        self._upload_test_document(label='second_doc')
        self.grant_access(
            obj=self.test_documents[1], permission=permission_document_view
        )
        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'q': 'first OR second'}, user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 2)
        self.assertTrue(self.test_documents[0] in queryset)
        self.assertTrue(self.test_documents[1] in queryset)

    def test_advanced_or_search(self):
        self._upload_test_document(label='first_doc')
        self.grant_access(
            obj=self.test_documents[0], permission=permission_document_view
        )

        self._upload_test_document(label='second_doc')
        self.grant_access(
            obj=self.test_documents[1], permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': 'first OR second'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 2)
        self.assertTrue(self.test_documents[0] in queryset)
        self.assertTrue(self.test_documents[1] in queryset)

    def test_simple_and_search(self):
        self._upload_test_document(label='second_doc')

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'q': 'non_valid second'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'q': 'second non_valid'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

    def test_simple_negated_search(self):
        self._upload_test_document(label='second_doc')

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'q': '-non_valid second'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 1)

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': '-second'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': '-second -Mayan'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': '-second OR -Mayan'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 1)

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': '-non_valid -second'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

    def test_search_with_dashed_content(self):
        self._upload_test_document(label='second-document')

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': '-second-document'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'label': '-"second-document"'},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 0)

    def test_search_field_transformation_functions(self):
        self._upload_test_document()

        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        queryset = self.search_backend.search(
            search_model=document_search,
            query_string={'uuid': force_text(self.test_document.uuid)},
            user=self._test_case_user
        )
        self.assertEqual(queryset.count(), 1)



import copy

from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.links import link_acl_list
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.navigation.classes import Link
from mayan.apps.navigation.utils import get_cascade_condition

from .permissions import (
    permission_cabinet_add_document, permission_cabinet_create,
    permission_cabinet_delete, permission_cabinet_edit,
    permission_cabinet_view, permission_cabinet_remove_document
)

# Document links

link_document_cabinet_list = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.cabinets.icons.icon_cabinet_list',
    permissions=(permission_document_view,),
    text=_('Cabinets'), view='cabinets:document_cabinet_list',
)
link_document_cabinet_remove = Link(
    args='resolved_object.pk',
    icon_class_path='mayan.apps.cabinets.icons.icon_document_cabinet_remove',
    permissions=(permission_cabinet_remove_document,),
    text=_('Remove from cabinets'), view='cabinets:document_cabinet_remove'
)
link_document_cabinet_add = Link(
    args='object.pk',
    icon_class_path='mayan.apps.cabinets.icons.icon_document_cabinet_add',
    permissions=(permission_cabinet_add_document,), text=_('Add to cabinets'),
    view='cabinets:document_cabinet_add',
)
link_document_multiple_cabinet_add = Link(
    icon_class_path='mayan.apps.cabinets.icons.icon_document_multiple_cabinet_add',
    text=_('Add to cabinets'), view='cabinets:document_multiple_cabinet_add'
)
link_multiple_document_cabinet_remove = Link(
    icon_class_path='mayan.apps.cabinets.icons.icon_document_cabinet_remove',
    text=_('Remove from cabinets'),
    view='cabinets:multiple_document_cabinet_remove'
)

# Cabinet links


def cabinet_is_root(context):
    return context[
        'resolved_object'
    ].is_root_node()


link_custom_acl_list = copy.copy(link_acl_list)
link_custom_acl_list.condition = cabinet_is_root

link_cabinet_child_add = Link(
    args='object.pk',
    icon_class_path='mayan.apps.cabinets.icons.icon_cabinet_child_add',
    permissions=(permission_cabinet_create,), text=_('Add new level'),
    view='cabinets:cabinet_child_add'
)
link_cabinet_create = Link(
    icon_class_path='mayan.apps.cabinets.icons.icon_cabinet_create',
    permissions=(permission_cabinet_create,),
    text=_('Create cabinet'), view='cabinets:cabinet_create'
)
link_cabinet_delete = Link(
    args='object.pk',
    icon_class_path='mayan.apps.cabinets.icons.icon_cabinet_delete',
    permissions=(permission_cabinet_delete,),
    tags='dangerous', text=_('Delete'), view='cabinets:cabinet_delete'
)
link_cabinet_edit = Link(
    args='object.pk',
    icon_class_path='mayan.apps.cabinets.icons.icon_cabinet_edit',
    permissions=(permission_cabinet_edit,), text=_('Edit'),
    view='cabinets:cabinet_edit'
)
link_cabinet_list = Link(
    condition=get_cascade_condition(
        app_label='cabinets', model_name='Cabinet',
        object_permission=permission_cabinet_view,
    ), icon_class_path='mayan.apps.cabinets.icons.icon_cabinet_list',
    text=_('All'), view='cabinets:cabinet_list'
)
link_cabinet_view = Link(
    args='object.pk', icon_class_path='mayan.apps.cabinets.icons.icon_cabinet_view',
    permissions=(permission_cabinet_view,), text=_('Details'),
    view='cabinets:cabinet_view'
)
